@@ -13,6 +13,8 @@
 
 #include <yoga/Yoga.h>
 
+QT_YOGA_NAMESPACE_BEGIN
+
 namespace facebook {
 namespace yoga {
 
@@ -25,4 +27,6 @@ void YGNodeToString(
 } // namespace yoga
 } // namespace facebook
 
+QT_YOGA_NAMESPACE_END
+
 #endif
