@@ -1,5 +1,6 @@
 // Copyright (C) 2019 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qqmltablemodel_p.h"
 
@@ -620,7 +621,7 @@ int QQmlTableModel::columnCount(const QModelIndex &parent) const
 */
 
 /*!
-    \qmlmethod bool TableModel::setData(QModelIndex index, string role, variant value)
+    \qmlmethod bool TableModel::setData(QModelIndex index, variant value, string role)
 
     Inserts or updates the data field named by \a role in the table cell at the
     given \a index with \a value. Returns true if sucessful, false if not.
