@@ -1,5 +1,6 @@
 // Copyright (C) 2025 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qqmltreemodel_p.h"
 #include "qqmltreerow_p.h"
@@ -514,7 +515,7 @@ int QQmlTreeModel::columnCount(const QModelIndex &parent) const
 */
 
 /*!
-    \qmlmethod bool TreeModel::setData(QModelIndex index, string role, variant value)
+    \qmlmethod bool TreeModel::setData(QModelIndex index, variant value, string role)
 
     Inserts or updates the data field named by \a role in the TreeRow at the
     given \a index with \a value. Returns true if sucessful, false if not.
