@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQml module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QtCore/qthreadstorage.h>
 
@@ -73,6 +37,9 @@ void QQmlAnimationTimer::unsetJobTimer(QAbstractAnimationJob *animation)
     if (animation->m_timer == this)
         animation->m_timer = nullptr;
 
+    if (animation->m_isPause)
+        runningPauseAnimations.removeOne(animation);
+
     if (animation->isGroup()) {
         QAnimationGroupJob *group = static_cast<QAnimationGroupJob *>(animation);
         if (const auto children = group->children()) {
@@ -84,12 +51,14 @@ void QQmlAnimationTimer::unsetJobTimer(QAbstractAnimationJob *animation)
 
 QQmlAnimationTimer::~QQmlAnimationTimer()
 {
-    for (const auto &animation : qAsConst(animations))
+    for (const auto &animation : std::as_const(animations))
         unsetJobTimer(animation);
-    for (const auto &animation : qAsConst(animationsToStart))
+    for (const auto &animation : std::as_const(animationsToStart))
         unsetJobTimer(animation);
-    for (const auto &animation : qAsConst(runningPauseAnimations))
+    for (const auto &animation : std::as_const(runningPauseAnimations))
         unsetJobTimer(animation);
+
+    QUnifiedTimer::stopAnimationTimer(this);
 }
 
 QQmlAnimationTimer *QQmlAnimationTimer::instance(bool create)
@@ -129,7 +98,7 @@ void QQmlAnimationTimer::updateAnimationsTime(qint64 delta)
     //when the CPU load is high
     if (delta) {
         insideTick = true;
-        for (currentAnimationIdx = 0; currentAnimationIdx < animations.count(); ++currentAnimationIdx) {
+        for (currentAnimationIdx = 0; currentAnimationIdx < animations.size(); ++currentAnimationIdx) {
             QAbstractAnimationJob *animation = animations.at(currentAnimationIdx);
             int elapsed = animation->m_totalCurrentTime
                           + (animation->direction() == QAbstractAnimationJob::Forward ? delta : -delta);
@@ -137,7 +106,7 @@ void QQmlAnimationTimer::updateAnimationsTime(qint64 delta)
         }
         if (animationTickDump()) {
             qDebug() << "***** Dumping Animation Tree ***** ( tick:" << lastTick << "delta:" << delta << ")";
-            for (int i = 0; i < animations.count(); ++i)
+            for (int i = 0; i < animations.size(); ++i)
                 qDebug() << animations.at(i);
         }
         insideTick = false;
@@ -243,16 +212,16 @@ void QQmlAnimationTimer::registerRunningAnimation(QAbstractAnimationJob *animati
 
 void QQmlAnimationTimer::unregisterRunningAnimation(QAbstractAnimationJob *animation)
 {
+    unsetJobTimer(animation);
     if (animation->userControlDisabled())
         return;
 
     if (animation->m_isGroup)
         return;
 
-    if (animation->m_isPause)
-        runningPauseAnimations.removeOne(animation);
-    else
+    if (!animation->m_isPause)
         runningLeafAnimations--;
+
     Q_ASSERT(runningLeafAnimations >= 0);
 }
 
@@ -307,9 +276,10 @@ QAbstractAnimationJob::~QAbstractAnimationJob()
 
         Q_ASSERT(m_state == Stopped);
         if (oldState == Running) {
-            Q_ASSERT(QQmlAnimationTimer::instance(false) == m_timer);
-            if (m_timer)
+            if (m_timer) {
+                Q_ASSERT(QQmlAnimationTimer::instance(false) == m_timer);
                 m_timer->unregisterAnimation(this);
+            }
         }
         Q_ASSERT(!m_hasRegisteredTimer);
     }
@@ -512,8 +482,11 @@ void QAbstractAnimationJob::setCurrentTime(int msecs)
 
     RETURN_IF_DELETED(updateCurrentTime(m_currentTime));
 
-    if (m_currentLoop != oldLoop)
-        currentLoopChanged();
+    if (m_currentLoop != oldLoop) {
+        // CurrentLoop listeners may restart the job if e.g. from has changed. Stopping a job will
+        // destroy it, so account for that here.
+        RETURN_IF_DELETED(currentLoopChanged());
+    }
 
     // All animations are responsible for stopping the animation when their
     // own end state is reached; in this case the animation is time driven,
@@ -551,6 +524,14 @@ void QAbstractAnimationJob::stop()
     setState(Stopped);
 }
 
+void QAbstractAnimationJob::complete()
+{
+    // Simulate the full animation cycle
+    setState(Running);
+    setCurrentTime(m_direction == Forward ? duration() : 0);
+    setState(Stopped);
+}
+
 void QAbstractAnimationJob::pause()
 {
     if (m_state == Stopped) {
