@@ -283,8 +283,9 @@ struct RequiredPropertyExtraData : public QV4::CompiledData::RequiredPropertyExt
 struct Function
 {
     QV4::CompiledData::Location location;
-    int nameIndex;
-    quint32 index; // index in parsedQML::functions
+    quint32 nameIndex : 31;
+    quint32 isQmlFunction : 1;
+    quint32 index = 0; // index in parsedQML::functions
     QQmlJS::FixedPoolArray<Parameter> formals;
     QV4::CompiledData::ParameterType returnType;
 
@@ -463,7 +464,7 @@ struct Q_QML_COMPILER_EXPORT Pragma
 
 struct Q_QML_COMPILER_EXPORT Document
 {
-    Document(bool debugMode);
+    Document(const QString &fileName, const QString &finalUrl, bool debugMode);
     QString code;
     QQmlJS::Engine jsParserEngine;
     QV4::Compiler::Module jsModule;
@@ -506,7 +507,7 @@ struct Q_QML_COMPILER_EXPORT IRBuilder : public QQmlJS::AST::Visitor
 {
     Q_DECLARE_TR_FUNCTIONS(QQmlCodeGenerator)
 public:
-    IRBuilder(const QSet<QString> &illegalNames);
+    IRBuilder();
     bool generateFromQml(const QString &code, const QString &url, Document *output);
 
     using QQmlJS::AST::Visitor::visit;
@@ -564,8 +565,8 @@ public:
     QStringView textRefAt(const QQmlJS::SourceLocation &first,
                          const QQmlJS::SourceLocation &last) const;
 
-    void setBindingValue(QV4::CompiledData::Binding *binding, QQmlJS::AST::Statement *statement,
-                         QQmlJS::AST::Node *parentNode);
+    virtual void setBindingValue(QV4::CompiledData::Binding *binding,
+                                 QQmlJS::AST::Statement *statement, QQmlJS::AST::Node *parentNode);
     void tryGeneratingTranslationBinding(QStringView base, QQmlJS::AST::ArgumentList *args, QV4::CompiledData::Binding *binding);
 
     void appendBinding(QQmlJS::AST::UiQualifiedId *name, QQmlJS::AST::Statement *value,
@@ -580,6 +581,9 @@ public:
 
     bool appendAlias(QQmlJS::AST::UiPublicMember *node);
 
+    enum class IsQmlFunction { Yes, No };
+    virtual void registerFunctionExpr(QQmlJS::AST::FunctionExpression *fexp, IsQmlFunction);
+
     Object *bindingsTarget() const;
 
     bool setId(const QQmlJS::SourceLocation &idLocation, QQmlJS::AST::Statement *value);
@@ -598,11 +602,10 @@ public:
     static bool isStatementNodeScript(QQmlJS::AST::Statement *statement);
     static bool isRedundantNullInitializerForPropertyDeclaration(Property *property, QQmlJS::AST::Statement *statement);
 
-    QString sanityCheckFunctionNames(Object *obj, const QSet<QString> &illegalNames, QQmlJS::SourceLocation *errorLocation);
+    QString sanityCheckFunctionNames(Object *obj, QQmlJS::SourceLocation *errorLocation);
 
     QList<QQmlJS::DiagnosticMessage> errors;
 
-    QSet<QString> illegalNames;
     QSet<QString> inlineComponentsNames;
 
     QList<const QV4::CompiledData::Import *> _imports;
@@ -632,7 +635,7 @@ private:
 
 struct Q_QML_COMPILER_EXPORT JSCodeGen : public QV4::Compiler::Codegen
 {
-    JSCodeGen(Document *document, const QSet<QString> &globalNames,
+    JSCodeGen(Document *document,
               QV4::Compiler::CodegenWarningInterface *iface =
                       QV4::Compiler::defaultCodegenWarningInterface(),
               bool storeSourceLocations = false);
@@ -765,6 +768,15 @@ void tryGeneratingTranslationBindingBase(QStringView base, QQmlJS::AST::Argument
         }
 
         args = args->next;
+        // QT_TR_NOOP can have a disambiguation string, QT_TRID_NOOP can't
+        if (args && base == QLatin1String("QT_TR_NOOP")) {
+            // we have a disambiguation string; we don't need to do anything with it
+            if (QQmlJS::AST::cast<QQmlJS::AST::StringLiteral *>(args->expression))
+                args = args->next;
+            else // second argument is not a string, stop
+                return;
+        }
+
         if (args)
             return; // too many arguments, stop
 
@@ -787,6 +799,14 @@ void tryGeneratingTranslationBindingBase(QStringView base, QQmlJS::AST::Argument
         }
 
         args = args->next;
+        if (args) {
+            // we have a disambiguation string; we don't need to do anything with it
+            if (QQmlJS::AST::cast<QQmlJS::AST::StringLiteral *>(args->expression))
+                args = args->next;
+            else // third argument is not a string, stop
+                return;
+        }
+
         if (args)
             return; // too many arguments, stop
 
