@@ -614,6 +614,9 @@ void QQmlEngine::clearComponentCache()
 {
     Q_D(QQmlEngine);
 
+    // QQmlGadgetPtrWrapper can have QQmlData with various references.
+    qDeleteAll(std::exchange(d->cachedValueTypeInstances, {}));
+
     // Contexts can hold on to CUs but live on the JS heap.
     // Use a non-incremental GC run to get rid of those.
     QV4::MemoryManager *mm = handle()->memoryManager;
@@ -985,7 +988,7 @@ void QQmlEngine::captureProperty(QObject *object, const QMetaProperty &property)
   \since 5.15
 
   The uiLanguage holds the name of the language to be used for user interface
-  string translations. It is exposed in C++ as QQmlEngine::uiLanguage property.
+  string translations. It is exposed in C++ as \l QJSEngine::uiLanguage property.
 
   You can set the value freely and use it in bindings. It is recommended to set it
   after installing translators in your application. By convention, an empty string
@@ -1936,7 +1939,7 @@ void QQmlEnginePrivate::executeRuntimeFunction(const QV4::ExecutableCompilationU
     Q_ASSERT(thisObject);
 
     QQmlData *ddata = QQmlData::get(thisObject);
-    Q_ASSERT(ddata && ddata->outerContext);
+    Q_ASSERT(ddata && ddata->context);
 
     QV4::Function *function = unit->runtimeFunctions[functionIndex];
     Q_ASSERT(function);
@@ -1951,7 +1954,7 @@ void QQmlEnginePrivate::executeRuntimeFunction(const QV4::ExecutableCompilationU
     QV4::Scope scope(v4);
     QV4::ExecutionContext *ctx = v4->scriptContext();
     QV4::Scoped<QV4::ExecutionContext> callContext(scope,
-        QV4::QmlContext::create(ctx, ddata->outerContext, thisObject));
+        QV4::QmlContext::create(ctx, ddata->context, thisObject));
 
     if (auto nested = function->nestedFunction()) {
         // if a nested function is already known, call the closure directly
