@@ -149,14 +149,31 @@ static QQmlTypePrivate *createQQmlType(QQmlMetaTypeData *data, const QString &el
     return d;
 }
 
-static void addQQmlMetaTypeInterfaces(QQmlTypePrivate *priv, const QByteArray &className)
+static void addQQmlMetaTypeInterfaces(
+        QQmlMetaTypeData *data, const QUrl &url, QQmlTypePrivate *priv, const QByteArray &className)
 {
     Q_ASSERT(!className.isEmpty());
     QByteArray ptr = className + '*';
     QByteArray lst = "QQmlListProperty<" + className + '>';
 
-    QMetaType ptr_type(new QQmlMetaTypeInterface(ptr));
-    QMetaType lst_type(new QQmlListMetaTypeInterface(lst, ptr_type.iface()));
+    QQmlMetaTypeData::CompositeMetaTypes &types = data->compositeMetaTypes[url];
+    if (types.type) {
+        Q_ASSERT(types.listType);
+
+        QMetaType::unregisterMetaType(QMetaType(types.type));
+        QMetaType::unregisterMetaType(QMetaType(types.listType));
+
+        types.type->name = std::move(ptr);
+        types.type->QMetaTypeInterface::name = types.type->name.constData();
+        types.listType->name = std::move(lst);
+        types.listType->QMetaTypeInterface::name = types.listType->name.constData();
+    } else {
+        types.type = new QQmlMetaTypeInterface(std::move(ptr));
+        types.listType = new QQmlListMetaTypeInterface(std::move(lst), types.type);
+    }
+
+    QMetaType ptr_type(types.type);
+    QMetaType lst_type(types.listType);
 
     // Retrieve the IDs once, so that the types are added to QMetaType's custom type registry.
     ptr_type.id();
@@ -177,7 +194,7 @@ static QQmlTypePrivate *createQQmlType(QQmlMetaTypeData *data, const QString &el
     const QUrl normalized = QQmlTypeLoader::normalize(type.url);
     d->extraData.compositeTypeData = normalized;
     addQQmlMetaTypeInterfaces(
-        d, QQmlPropertyCacheCreatorBase::createClassNameTypeByUrl(normalized));
+            data, normalized, d, QQmlPropertyCacheCreatorBase::createClassNameTypeByUrl(normalized));
     return d;
 }
 
@@ -193,8 +210,9 @@ static QQmlTypePrivate *createQQmlType(
     d->version = type.version;
 
     d->extraData.singletonTypeData->singletonInstanceInfo = siinfo;
+    const QUrl &url = siinfo->url;
     addQQmlMetaTypeInterfaces(
-            d, QQmlPropertyCacheCreatorBase::createClassNameTypeByUrl(siinfo->url));
+            data, url, d, QQmlPropertyCacheCreatorBase::createClassNameTypeByUrl(url));
     return d;
 }
 
@@ -307,15 +325,23 @@ void QQmlMetaType::clearTypeRegistrations()
     data->nameToType.clear();
     data->urlToType.clear();
     data->typePropertyCaches.clear();
-    data->urlToNonFileImportType.clear();
     data->metaObjectToType.clear();
     data->undeletableTypes.clear();
     data->propertyCaches.clear();
-    data->inlineComponentTypes.clear();
+
+    qDeleteAll(data->metaTypeToValueType);
+    data->metaTypeToValueType.clear();
+
+    data->moduleImports.clear();
 
     // Avoid deletion recursion (via QQmlTypePrivate dtor) by moving them out of the way first.
     QQmlMetaTypeData::CompositeTypes emptyComposites;
     emptyComposites.swap(data->compositeTypes);
+
+    qDeleteAll(data->metaTypeToValueType);
+    data->metaTypeToValueType.clear();
+
+    data->clearCompositeMetaTypes();
 }
 
 void QQmlMetaType::registerTypeAlias(int typeIndex, const QString &name)
@@ -522,19 +548,15 @@ QQmlType QQmlMetaType::registerCompositeSingletonType(
     QQmlMetaTypeDataPtr data;
 
     QString typeName = QString::fromUtf8(type.typeName);
-    bool fileImport = false;
-    if (*(type.uri) == '\0')
-        fileImport = true;
-    if (!checkRegistration(QQmlType::CompositeSingletonType, data, fileImport ? nullptr : type.uri,
-                           typeName, type.version, {})) {
+    if (!checkRegistration(
+                QQmlType::CompositeSingletonType, data, type.uri, typeName, type.version, {})) {
         return QQmlType();
     }
 
     QQmlTypePrivate *priv = createQQmlType(data, typeName, type, siinfo);
     addTypeToData(priv, data);
 
-    QQmlMetaTypeData::Files *files = fileImport ? &(data->urlToType) : &(data->urlToNonFileImportType);
-    files->insert(siinfo->url, priv);
+    data->urlToType.insert(siinfo->url, priv);
 
     return QQmlType(priv);
 }
@@ -548,19 +570,13 @@ QQmlType QQmlMetaType::registerCompositeType(const QQmlPrivate::RegisterComposit
     QQmlMetaTypeDataPtr data;
 
     QString typeName = QString::fromUtf8(type.typeName);
-    bool fileImport = false;
-    if (*(type.uri) == '\0')
-        fileImport = true;
-    if (!checkRegistration(QQmlType::CompositeType, data, fileImport?nullptr:type.uri, typeName,
-                           type.version, {})) {
+    if (!checkRegistration(QQmlType::CompositeType, data, type.uri, typeName, type.version, {}))
         return QQmlType();
-    }
 
     QQmlTypePrivate *priv = createQQmlType(data, typeName, type);
     addTypeToData(priv, data);
 
-    QQmlMetaTypeData::Files *files = fileImport ? &(data->urlToType) : &(data->urlToNonFileImportType);
-    files->insert(QQmlTypeLoader::normalize(type.url), priv);
+    data->urlToType.insert(QQmlTypeLoader::normalize(type.url), priv);
 
     return QQmlType(priv);
 }
@@ -636,7 +652,8 @@ static QQmlType createTypeForUrl(
         //       and shared across threads.
 
         auto *priv = new QQmlTypePrivate(registrationType);
-        addQQmlMetaTypeInterfaces(priv, QQmlPropertyCacheCreatorBase::createClassNameTypeByUrl(url));
+        addQQmlMetaTypeInterfaces(
+                data, url, priv, QQmlPropertyCacheCreatorBase::createClassNameTypeByUrl(url));
 
         priv->setName(QString(), typeName);
         priv->version = version;
@@ -688,11 +705,8 @@ QQmlType QQmlMetaType::findCompositeType(
 
     bool urlExists = true;
     auto found = data->urlToType.constFind(normalized);
-    if (found == data->urlToType.cend()) {
-        found = data->urlToNonFileImportType.constFind(normalized);
-        if (found == data->urlToNonFileImportType.cend())
-            urlExists = false;
-    }
+    if (found == data->urlToType.cend())
+        urlExists = false;
 
     if (urlExists) {
         if (compilationUnit.isNull())
@@ -717,17 +731,16 @@ static QQmlType doRegisterInlineComponentType(QQmlMetaTypeData *data, const QUrl
     priv->setName(QString(), url.fragment());
 
     priv->extraData.inlineComponentTypeData = url;
+    data->registerType(priv);
 
-    const QByteArray className
-            = QQmlPropertyCacheCreatorBase::createClassNameForInlineComponent(url, url.fragment());
+    addQQmlMetaTypeInterfaces(
+            data, url, priv, QQmlPropertyCacheCreatorBase::createClassNameForInlineComponent(url));
+    data->urlToType.insert(url, priv);
 
-    addQQmlMetaTypeInterfaces(priv, className);
-    const QQmlType result(priv);
-    priv->release();
+    data->idToType.insert(priv->typeId.id(), priv);
+    data->idToType.insert(priv->listId.id(), priv);
 
-    data->inlineComponentTypes.insert(url, result);
-
-    return result;
+    return QQmlType(priv);
 }
 
 QQmlType QQmlMetaType::findInlineComponentType(
@@ -737,37 +750,16 @@ QQmlType QQmlMetaType::findInlineComponentType(
 
     // If there is an "unclaimed" inline component type, we can "claim" it now. Otherwise
     // we have to create a new one.
-    const auto it = data->inlineComponentTypes.constFind(url);
-    if (it != data->inlineComponentTypes.constEnd()) {
-        const auto jt = data->compositeTypes.constFind(it->typeId().iface());
+    const auto it = data->urlToType.constFind(url);
+    if (it != data->urlToType.constEnd()) {
+        const auto jt = data->compositeTypes.constFind((*it)->typeId.iface());
         if (jt == data->compositeTypes.constEnd() || *jt == compilationUnit)
-            return *it;
+            return QQmlType(*it);
     }
 
     return doRegisterInlineComponentType(data, url);
 }
 
-void QQmlMetaType::unregisterInternalCompositeType(QMetaType metaType, QMetaType listMetaType)
-{
-    // This may be called from delayed dtors on shutdown when the data is already gone.
-    QQmlMetaTypeDataPtr data;
-    if (data.isValid()) {
-        if (QQmlValueType *vt = data->metaTypeToValueType.take(metaType.id()))
-            delete vt;
-        if (QQmlValueType *vt = data->metaTypeToValueType.take(listMetaType.id()))
-            delete vt;
-
-        auto it = data->compositeTypes.constFind(metaType.iface());
-        if (it != data->compositeTypes.constEnd())
-            data->compositeTypes.erase(it);
-    }
-
-    QMetaType::unregisterMetaType(metaType);
-    QMetaType::unregisterMetaType(listMetaType);
-    delete static_cast<const QQmlMetaTypeInterface *>(metaType.iface());
-    delete static_cast<const QQmlListMetaTypeInterface *>(listMetaType.iface());
-}
-
 int QQmlMetaType::registerUnitCacheHook(
         const QQmlPrivate::RegisterQmlUnitCacheHook &hookRegistration)
 {
@@ -1045,11 +1037,6 @@ QQmlType QQmlMetaType::typeForUrl(const QString &urlString,
         if (ret.isValid() && ret.sourceUrl() == url)
             return ret;
     }
-    {
-        QQmlType ret(data->urlToNonFileImportType.value(url));
-        if (ret.isValid() && ret.sourceUrl() == url)
-            return ret;
-    }
 
     const QQmlType type = createTypeForUrl(
         data, url, qualifiedType, mode, errors, version);
@@ -1361,14 +1348,12 @@ QQmlType QQmlMetaType::qmlListType(QMetaType metaType)
 
     Returns null if no such type is registered.
 */
-QQmlType QQmlMetaType::qmlType(const QUrl &unNormalizedUrl, bool includeNonFileImports /* = false */)
+QQmlType QQmlMetaType::qmlType(const QUrl &unNormalizedUrl)
 {
     const QUrl url = QQmlTypeLoader::normalize(unNormalizedUrl);
     const QQmlMetaTypeDataPtr data;
 
     QQmlType type(data->urlToType.value(url));
-    if (!type.isValid() && includeNonFileImports)
-        type = QQmlType(data->urlToNonFileImportType.value(url));
 
     if (type.sourceUrl() == url)
         return type;
@@ -1379,9 +1364,9 @@ QQmlType QQmlMetaType::qmlType(const QUrl &unNormalizedUrl, bool includeNonFileI
 QQmlType QQmlMetaType::fetchOrCreateInlineComponentTypeForUrl(const QUrl &url)
 {
     QQmlMetaTypeDataPtr data;
-    const auto it = data->inlineComponentTypes.constFind(url);
-    if (it != data->inlineComponentTypes.constEnd())
-        return *it;
+    const auto it = data->urlToType.constFind(url);
+    if (it != data->urlToType.constEnd())
+        return QQmlType(*it);
 
     return doRegisterInlineComponentType(data, url);
 }
@@ -1527,11 +1512,10 @@ void QQmlMetaType::unregisterType(int typeIndex)
     const QQmlType type = data->types.value(typeIndex);
     if (const QQmlTypePrivate *d = type.priv()) {
         if (d->regType == QQmlType::CompositeType || d->regType == QQmlType::CompositeSingletonType)
-            removeFromInlineComponents(data->inlineComponentTypes, d);
+            removeFromInlineComponents(data->urlToType, d);
         removeQQmlTypePrivate(data->idToType, d);
         removeQQmlTypePrivate(data->nameToType, d);
         removeQQmlTypePrivate(data->urlToType, d);
-        removeQQmlTypePrivate(data->urlToNonFileImportType, d);
         removeQQmlTypePrivate(data->metaObjectToType, d);
         for (auto & module : data->uriToModule)
             module->remove(d);
@@ -1551,12 +1535,12 @@ void QQmlMetaType::registerMetaObjectForType(const QMetaObject *metaobject, QQml
 
 static bool hasActiveInlineComponents(const QQmlMetaTypeData *data, const QQmlTypePrivate *d)
 {
-    for (auto it = data->inlineComponentTypes.begin(), end = data->inlineComponentTypes.end();
+    for (auto it = data->urlToType.begin(), end = data->urlToType.end();
          it != end; ++it) {
         if (!QQmlMetaType::equalBaseUrls(it.key(), d->sourceUrl()))
             continue;
 
-        const QQmlTypePrivate *icPriv = it->priv();
+        const QQmlTypePrivate *icPriv = *it;
         if (icPriv && icPriv->count() > 1)
             return true;
     }
@@ -1617,12 +1601,11 @@ void QQmlMetaType::freeUnusedTypesAndCaches()
 
                 if (d->regType == QQmlType::CompositeType
                         || d->regType == QQmlType::CompositeSingletonType) {
-                    removeFromInlineComponents(data->inlineComponentTypes, d);
+                    removeFromInlineComponents(data->urlToType, d);
                 }
                 removeQQmlTypePrivate(data->idToType, d);
                 removeQQmlTypePrivate(data->nameToType, d);
                 removeQQmlTypePrivate(data->urlToType, d);
-                removeQQmlTypePrivate(data->urlToNonFileImportType, d);
                 removeQQmlTypePrivate(data->metaObjectToType, d);
 
                 for (auto &module : data->uriToModule)
@@ -1888,6 +1871,9 @@ static bool isInternalType(int idx)
 
 bool QQmlMetaType::isValueType(QMetaType type)
 {
+    if (type.flags().testFlag(QMetaType::PointerToQObject))
+        return false;
+
     if (!type.isValid() || isInternalType(type.id()))
         return false;
 
@@ -2015,11 +2001,8 @@ QQmlRefPointer<QV4::CompiledData::CompilationUnit> QQmlMetaType::obtainCompilati
     QQmlMetaTypeDataPtr data;
 
     auto found = data->urlToType.constFind(normalized);
-    if (found == data->urlToType.constEnd()) {
-        found = data->urlToNonFileImportType.constFind(normalized);
-        if (found == data->urlToNonFileImportType.constEnd())
-            return QQmlRefPointer<QV4::CompiledData::CompilationUnit>();
-    }
+    if (found == data->urlToType.constEnd())
+        return QQmlRefPointer<QV4::CompiledData::CompilationUnit>();
 
     const auto composite = data->compositeTypes.constFind(found.value()->typeId.iface());
     return composite == data->compositeTypes.constEnd()
