@@ -732,7 +732,10 @@ QQmlError QQmlPropertyValidator::validateObjectBinding(const QQmlPropertyData *p
         if (!QQmlMetaType::isInterface(listType)) {
             QQmlPropertyCache::ConstPtr source = propertyCaches.at(binding->value.objectIndex);
             if (!canCoerce(listType, source)) {
-                return qQmlCompileError(binding->valueLocation, tr("Cannot assign object to list property \"%1\"").arg(propertyName));
+                const QString expectedTypeName = QString::fromUtf8(listType.name()).remove(QLatin1Char('*'));
+                return qQmlCompileError(binding->valueLocation,
+                    tr("Cannot assign object of type \"%1\" to list property \"%2\"; expected \"%3\"")
+                        .arg(source->className(), propertyName, expectedTypeName));
             }
         }
         return noError;
@@ -748,26 +751,27 @@ QQmlError QQmlPropertyValidator::validateObjectBinding(const QQmlPropertyData *p
     } else if (propType == QMetaType::fromType<QQmlScriptString>()) {
         return qQmlCompileError(binding->valueLocation, tr("Invalid property assignment: script expected"));
     } else if (!QQmlMetaType::isValueType(property->propType())) {
-        // We want to use the raw metaObject here as the raw metaobject is the
-        // actual property type before we applied any extensions that might
-        // effect the properties on the type, but don't effect assignability
-        // Not passing a version ensures that we get the raw metaObject.
-        QQmlPropertyCache::ConstPtr propertyMetaObject
-                = QQmlMetaType::rawPropertyCacheForType(propType);
-        if (!propertyMetaObject) {
-            // if we have an inline component from the current file,
-            // it is not properly registered at this point, as registration
-            // only occurs after the whole file has been validated
-            // Therefore we need to check the ICs here
-            for (const auto& icDatum: compilationUnit->inlineComponentData) {
-                if (icDatum.qmlType.typeId() == property->propType()) {
-                    propertyMetaObject
-                            = compilationUnit->propertyCaches.at(icDatum.objectIndex);
-                    break;
-                }
+        QQmlPropertyCache::ConstPtr propertyMetaObject;
+
+        // if we have an inline component from the current file,
+        // it is not properly registered at this point, as registration
+        // only occurs after the whole file has been validated
+        // Therefore we need to check the ICs here
+        for (const auto &icDatum: std::as_const(compilationUnit->inlineComponentData)) {
+            if (icDatum.qmlType.typeId() == property->propType()) {
+                propertyMetaObject = compilationUnit->propertyCaches.at(icDatum.objectIndex);
+                break;
             }
         }
 
+        if (!propertyMetaObject) {
+            // We want to use the raw metaObject here as the raw metaobject is the
+            // actual property type before we applied any extensions that might
+            // effect the properties on the type, but don't effect assignability
+            // Not passing a version ensures that we get the raw metaObject.
+            propertyMetaObject = QQmlMetaType::rawPropertyCacheForType(propType);
+        }
+
         if (propertyMetaObject) {
             // Will be true if the assigned type inherits propertyMetaObject
             // Determine isAssignable value
