@@ -6,8 +6,8 @@
 
 QT_BEGIN_NAMESPACE
 
-QQmlProxyMetaObject::QQmlProxyMetaObject(QObject *obj, QList<ProxyData> *mList)
-: metaObjects(mList), proxies(nullptr), parent(nullptr), object(obj)
+QQmlProxyMetaObject::QQmlProxyMetaObject(QObject *obj, const QList<ProxyData> *mList)
+    : metaObjects(mList), proxies(nullptr), parent(nullptr), object(obj)
 {
     metaObject = metaObjects->constFirst().metaObject;
 
@@ -127,9 +127,12 @@ int QQmlProxyMetaObject::metaCall(QObject *o, QMetaObject::Call c, int id, void
         return object->qt_metacall(c, id, a);
 }
 
-QMetaObject *QQmlProxyMetaObject::toDynamicMetaObject(QObject *)
+void QQmlProxyMetaObject::objectDestroyed(QObject *object)
 {
-    return metaObject;
+    if (parent)
+        parent->objectDestroyed(object);
+    else
+        QDynamicMetaObjectData::objectDestroyed(object);
 }
 
 QT_END_NAMESPACE
