@@ -1,5 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
+// Qt-Security score:significant
 
 #include "qqmljstyperesolver_p.h"
 
@@ -109,6 +110,9 @@ QQmlJSTypeResolver::QQmlJSTypeResolver(QQmlJSImporter *importer)
     m_varType = builtinTypes.type(u"QVariant"_s).scope;
     Q_ASSERT(m_varType);
 
+    m_qmlPropertyMapType = builtinTypes.type(u"QQmlPropertyMap"_s).scope;
+    Q_ASSERT(m_qmlPropertyMapType);
+
     m_jsValueType = builtinTypes.type(u"QJSValue"_s).scope;
     Q_ASSERT(m_jsValueType);
 
@@ -360,12 +364,49 @@ bool QQmlJSTypeResolver::isPrimitive(const QQmlJSScope::ConstPtr &type) const
 
 bool QQmlJSTypeResolver::isNumeric(const QQmlJSScope::ConstPtr &type) const
 {
-    return QQmlJSUtils::searchBaseAndExtensionTypes(
-            type, [&](const QQmlJSScope::ConstPtr &scope, QQmlJSScope::ExtensionKind mode) {
-                if (mode == QQmlJSScope::ExtensionNamespace)
-                    return false;
-                return scope == m_numberPrototype;
-    });
+    if (!type) // should this be a precondition instead?
+        return false;
+
+    if (type->scopeType() == QQmlJSScope::ScopeType::EnumScope)
+        return true;
+
+    if (type == m_realType)
+        return true;
+    if (type == m_floatType)
+        return true;
+    if (type == m_int8Type)
+        return true;
+    if (type == m_uint8Type)
+        return true;
+    if (type == m_int16Type)
+        return true;
+    if (type == m_uint16Type)
+        return true;
+    if (type == m_int32Type)
+        return true;
+    if (type == m_uint32Type)
+        return true;
+    if (type == m_int64Type)
+        return true;
+    if (type == m_uint64Type)
+        return true;
+    // sizetype is covered by one of the above cases (int32 or int64)
+    // booleans are not numeric
+
+    // the number prototype is numeric
+    if (type == m_numberPrototype)
+        return true;
+
+    // and types directly inheriting from it, notably number, are also
+    // numeric
+    if (type->baseType() == m_numberPrototype)
+        return true;
+
+    // it isn't possible (for users) to derive from m_numberPrototpye,
+    // so we know the list above is exhaustive / we don't need to go up
+    // further in the inheritance chain
+
+    return false;
 }
 
 bool QQmlJSTypeResolver::isSignedInteger(const QQmlJSScope::ConstPtr &type) const
@@ -496,16 +537,28 @@ bool QQmlJSTypeResolver::adjustTrackedType(
         return true;
 
     QQmlJSScope::ConstPtr contained = tracked.containedType();
+    QQmlJSScope::ConstPtr result = conversion;
+
+    // Do not adjust to the JavaScript extension of the original type. Rather keep the original
+    // type in that case.
+    QQmlJSUtils::searchBaseAndExtensionTypes(
+            contained, [&](const QQmlJSScope::ConstPtr &scope, QQmlJSScope::ExtensionKind kind) {
+        if (kind != QQmlJSScope::ExtensionJavaScript || scope != result)
+            return false;
+        result = contained;
+        return true;
+    });
+
 
     // If we cannot convert to the new type without the help of e.g. lookupResultMetaType(),
     // we better not change the type.
-    if (!canPrimitivelyConvertFromTo(contained, conversion)
-        && !canPopulate(conversion, contained, nullptr)
-        && !selectConstructor(conversion, contained, nullptr).isValid()) {
+    if (!canPrimitivelyConvertFromTo(contained, result)
+        && !canPopulate(result, contained, nullptr)
+        && !selectConstructor(result, contained, nullptr).isValid()) {
         return false;
     }
 
-    m_pool->adjustType(tracked, conversion);
+    m_pool->adjustType(tracked, result);
     return true;
 }
 
@@ -525,18 +578,7 @@ bool QQmlJSTypeResolver::adjustTrackedType(
     for (QQmlJSRegisterContent type : conversions)
         result = merge(type.containedType(), result);
 
-    QQmlJSScope::ConstPtr contained = tracked.containedType();
-
-    // If we cannot convert to the new type without the help of e.g. lookupResultMetaType(),
-    // we better not change the type.
-    if (!canPrimitivelyConvertFromTo(contained, result)
-            && !canPopulate(result, contained, nullptr)
-            && !selectConstructor(result, contained, nullptr).isValid()) {
-        return false;
-    }
-
-    m_pool->adjustType(tracked, result);
-    return true;
+    return adjustTrackedType(tracked, result);
 }
 
 void QQmlJSTypeResolver::adjustOriginalType(
@@ -654,19 +696,37 @@ QQmlJSScope::ConstPtr QQmlJSTypeResolver::merge(const QQmlJSScope::ConstPtr &a,
     if (b.isNull())
         return a;
 
-    const auto commonBaseType = [](
-            const QQmlJSScope::ConstPtr &a, const QQmlJSScope::ConstPtr &b) {
-        for (QQmlJSScope::ConstPtr aBase = a; aBase; aBase = aBase->baseType()) {
-            for (QQmlJSScope::ConstPtr bBase = b; bBase; bBase = bBase->baseType()) {
-                if (aBase == bBase)
-                    return aBase;
+    const auto baseOrExtension
+            = [](const QQmlJSScope::ConstPtr &a, const QQmlJSScope::ConstPtr &b) {
+        QQmlJSScope::ConstPtr found;
+        QQmlJSUtils::searchBaseAndExtensionTypes(
+                a, [&](const QQmlJSScope::ConstPtr &scope, QQmlJSScope::ExtensionKind kind) {
+            switch (kind) {
+            case QQmlJSScope::NotExtension:
+                // If b inherits scope, then scope is a common base type of a and b
+                if (b->inherits(scope)) {
+                    found = scope;
+                    return true;
+                }
+                break;
+            case QQmlJSScope::ExtensionJavaScript:
+                // Merging a type with its JavaScript extension produces the type.
+                // Giving the JavaScript extension as type to be read means we expect any type
+                // that fulfills the given JavaScript interface
+                if (scope == b) {
+                    found = a;
+                    return true;
+                }
+                break;
+            case QQmlJSScope::ExtensionType:
+            case QQmlJSScope::ExtensionNamespace:
+                break;
             }
-        }
-
-        return QQmlJSScope::ConstPtr();
+            return false;
+        });
+        return found;
     };
 
-
     if (a == b)
         return a;
 
@@ -699,8 +759,11 @@ QQmlJSScope::ConstPtr QQmlJSTypeResolver::merge(const QQmlJSScope::ConstPtr &a,
     if (isPrimitive(a) && isPrimitive(b))
         return jsPrimitiveType();
 
-    if (auto commonBase = commonBaseType(a, b))
-        return commonBase;
+    if (const auto base = baseOrExtension(a, b))
+        return base;
+
+    if (const auto base = baseOrExtension(b, a))
+        return base;
 
     if ((a == nullType() || a == boolType()) && b->isReferenceType())
         return b;
@@ -985,22 +1048,6 @@ QQmlJSScope::ConstPtr QQmlJSTypeResolver::resolveParentProperty(
     return propType;
 }
 
-/*!
- * \internal
- * We can generally determine the relevant component boundaries for each scope. However,
- * if the scope or any of its parents is assigned to a property of which we cannot see the
- * type, we don't know whether the type of that property happens to be Component. In that
- * case, we can't say.
- */
-bool QQmlJSTypeResolver::canFindComponentBoundaries(const QQmlJSScope::ConstPtr &scope) const
-{
-    for (QQmlJSScope::ConstPtr parent = scope; parent; parent = parent->parentScope()) {
-        if (parent->isAssignedToUnknownProperty())
-            return false;
-    }
-    return true;
-}
-
 /*!
  * \internal
  *
@@ -1035,11 +1082,17 @@ QQmlJSScope::ConstPtr QQmlJSTypeResolver::scopedType(
         const QQmlJSScope::ConstPtr &scope, const QString &name,
         QQmlJSScopesByIdOptions options) const
 {
-    if (!canFindComponentBoundaries(scope))
+    QQmlJSScopesById::CertainCallback<QQmlJSScope::ConstPtr> identified;
+    if (m_objectsById.possibleScopes(name, scope, options, identified)
+            != QQmlJSScopesById::Success::Yes) {
+        // Could not determine component boundaries
         return {};
+    }
 
-    if (QQmlJSScope::ConstPtr identified = m_objectsById.scope(name, scope, options))
-        return identified;
+    if (identified.result) {
+        // Found a definite match
+        return identified.result;
+    }
 
     if (QQmlJSScope::ConstPtr base = QQmlJSScope::findCurrentQMLScope(scope)) {
         QQmlJSScope::ConstPtr result;
@@ -1097,12 +1150,18 @@ QQmlJSRegisterContent QQmlJSTypeResolver::scopedType(QQmlJSRegisterContent scope
                                                      QQmlJSScopesByIdOptions options) const
 {
     const QQmlJSScope::ConstPtr contained = scope.containedType();
-    if (!canFindComponentBoundaries(contained))
-        return {};
 
-    if (QQmlJSScope::ConstPtr identified = m_objectsById.scope(name, contained, options)) {
+    QQmlJSScopesById::CertainCallback<QQmlJSScope::ConstPtr> identified;
+    if (m_objectsById.possibleScopes(name, contained, options, identified)
+            != QQmlJSScopesById::Success::Yes) {
+        // Could not determine component boundaries
+        return {};
+    }
+
+    if (identified.result) {
+        // Found a definite match
         return m_pool->createType(
-                identified, lookupIndex, QQmlJSRegisterContent::ObjectById, scope);
+                identified.result, lookupIndex, QQmlJSRegisterContent::ObjectById, scope);
     }
 
     if (QQmlJSScope::ConstPtr base = QQmlJSScope::findCurrentQMLScope(contained)) {
@@ -1422,18 +1481,31 @@ bool QQmlJSTypeResolver::canPrimitivelyConvertFromTo(
     if (to == m_jsPrimitiveType)
         return isPrimitive(from);
 
-    if (from == m_variantListType)
-        return to->accessSemantics() == QQmlJSScope::AccessSemantics::Sequence;
-
     const bool matchByName = !to->isComposite();
     Q_ASSERT(!matchByName || !to->internalName().isEmpty());
-    for (auto baseType = from; baseType; baseType = baseType->baseType()) {
-        if (baseType == to)
-            return true;
-        if (matchByName && baseType->internalName() == to->internalName())
-            return true;
+    if (QQmlJSUtils::searchBaseAndExtensionTypes(
+            from, [&](const QQmlJSScope::ConstPtr &scope, QQmlJSScope::ExtensionKind kind) {
+        switch (kind) {
+        case QQmlJSScope::NotExtension:
+        case QQmlJSScope::ExtensionJavaScript:
+            // Converting to a base type is trivially supported.
+            // Converting to the JavaScript extension of a type just produces the type itself.
+            // Giving the JavaScript extension as type to be converted to means we expect any
+            // result that fulfills the given JavaScript interface.
+            return scope == to
+                    || (matchByName && scope->internalName() == to->internalName());
+        case QQmlJSScope::ExtensionType:
+        case QQmlJSScope::ExtensionNamespace:
+            break;
+        }
+        return false;
+    })) {
+        return true;
     }
 
+    if (from == m_variantListType)
+        return to->accessSemantics() == QQmlJSScope::AccessSemantics::Sequence;
+
     // We can convert anything that fits into QJSPrimitiveValue
     if (canConvertFromTo(from, m_jsPrimitiveType) && canConvertFromTo(m_jsPrimitiveType, to))
         return true;
@@ -1451,6 +1523,12 @@ bool QQmlJSTypeResolver::canPrimitivelyConvertFromTo(
         return true;
     }
 
+    // it is possible to assing a singlar object to a list property if it could be stored in the list
+    if (to->accessSemantics() == QQmlJSScope::AccessSemantics::Sequence
+        && from->accessSemantics()  == QQmlJSScope::AccessSemantics::Reference
+        &&  from->inherits(to->valueType()))
+        return true;
+
     if (to == m_stringType && from->accessSemantics() == QQmlJSScope::AccessSemantics::Sequence)
         return canConvertFromTo(from->valueType(), m_stringType);
 
@@ -1479,9 +1557,9 @@ QQmlJSRegisterContent QQmlJSTypeResolver::memberType(
 
     // If we got a plain type reference we have to check the enums of the _scope_.
     if (contained == metaObjectType())
-        return {};
+        return memberEnumType(type.scope(), name);
 
-    if (contained == variantMapType()) {
+    if (contained == variantMapType() || contained->inherits(qmlPropertyMapType())) {
         QQmlJSMetaProperty prop;
         prop.setPropertyName(name);
         prop.setTypeName(u"QVariant"_s);
@@ -1555,6 +1633,12 @@ QQmlJSRegisterContent QQmlJSTypeResolver::memberType(
         }
     }
 
+    // check enums before checking attached types of attached types (chained attached types)
+    if (type.isType()) {
+        if (auto result = memberEnumType(type.scope(), name); result.isValid())
+            return result;
+    }
+
     if (QQmlJSScope::ConstPtr attachedBase = typeForName(name)) {
         if (QQmlJSScope::ConstPtr attached = attachedBase->attachedType()) {
             if (!genericType(attached)) {
@@ -1604,15 +1688,8 @@ QQmlJSRegisterContent QQmlJSTypeResolver::memberEnumType(
 QQmlJSRegisterContent QQmlJSTypeResolver::memberType(
         QQmlJSRegisterContent type, const QString &name, int lookupIndex) const
 {
-    if (type.isType()) {
-        const auto result = memberType(type, name, type.resultLookupIndex(), lookupIndex);
-        if (result.isValid())
-            return result;
-
-        // If we didn't find anything and it's an attached type,
-        // we might have an enum of the attaching type.
-        return memberEnumType(type.scope(), name);
-    }
+    if (type.isType())
+        return memberType(type, name, type.resultLookupIndex(), lookupIndex);
     if (type.isProperty() || type.isMethodCall())
         return memberType(type, name, type.resultLookupIndex(), lookupIndex);
     if (type.isEnumeration()) {
