@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickmousearea_p.h"
 #include "qquickmousearea_p_p.h"
@@ -21,8 +22,6 @@ QT_BEGIN_NAMESPACE
 
 DEFINE_BOOL_CONFIG_OPTION(qmlMaVisualTouchDebugging, QML_VISUAL_TOUCH_DEBUGGING)
 
-Q_DECLARE_LOGGING_CATEGORY(lcHoverTrace)
-
 QQuickMouseAreaPrivate::QQuickMouseAreaPrivate()
 : enabled(true), hoverEnabled(false), scrollGestureEnabled(true), hovered(false), longPress(false),
   moved(false), stealMouse(false), doubleClick(false), preventStealing(false),
@@ -161,7 +160,7 @@ bool QQuickMouseAreaPrivate::propagateHelper(QQuickMouseEvent *ev, QQuickItem *i
 
 /*!
     \qmltype MouseArea
-    \instantiates QQuickMouseArea
+    \nativetype QQuickMouseArea
     \inqmlmodule QtQuick
     \ingroup qtquick-input
     \brief Enables simple mouse handling.
@@ -743,7 +742,9 @@ void QQuickMouseArea::mouseMoveEvent(QMouseEvent *event)
 
     QQuickMouseEvent &me = d->quickMouseEvent;
     me.reset(d->lastPos.x(), d->lastPos.y(), d->lastButton, d->lastButtons, d->lastModifiers, false, d->longPress, event->flags());
+#if QT_DEPRECATED_SINCE(6, 6)
     me.setSource(event->source());
+#endif
     emit mouseXChanged(&me);
     me.setPosition(d->lastPos);
     emit mouseYChanged(&me);
@@ -788,7 +789,9 @@ void QQuickMouseArea::mouseDoubleClickEvent(QMouseEvent *event)
         QQuickMouseEvent &me = d->quickMouseEvent;
         me.reset(d->lastPos.x(), d->lastPos.y(), d->lastButton, d->lastButtons, d->lastModifiers, true,
                  false, event->flags());
+#if QT_DEPRECATED_SINCE(6, 6)
         me.setSource(event->source());
+#endif
         me.setAccepted(d->isDoubleClickConnected());
         emit this->doubleClicked(&me);
         if (!me.isAccepted())
@@ -825,6 +828,7 @@ void QQuickMouseArea::hoverEnterEvent(QHoverEvent *event)
         me.setPosition(d->lastPos);
         emit mouseYChanged(&me);
         me.setPosition(d->lastPos);
+        emit positionChanged(&me);
     }
 
     // A MouseArea should not block hover events
@@ -905,6 +909,7 @@ void QQuickMouseArea::ungrabMouse()
         emit pressedButtonsChanged();
 
         if (d->hovered && !isUnderMouse()) {
+            qCDebug(lcHoverTrace) << "losing hover: not under the mouse";
             d->hovered = false;
             emit hoveredChanged();
         }
@@ -969,6 +974,7 @@ bool QQuickMouseArea::sendMouseEvent(QMouseEvent *event)
                 emit pressedChanged();
                 emit containsPressChanged();
                 if (d->hovered) {
+                    qCDebug(lcHoverTrace) << "losing hover: button released";
                     d->hovered = false;
                     emit hoveredChanged();
                 }
@@ -1015,7 +1021,9 @@ void QQuickMouseArea::timerEvent(QTimerEvent *event)
             d->longPress = true;
             QQuickMouseEvent &me = d->quickMouseEvent;
             me.reset(d->lastPos.x(), d->lastPos.y(), d->lastButton, d->lastButtons, d->lastModifiers, false, d->longPress, d->lastFlags);
+#if QT_DEPRECATED_SINCE(6, 6)
             me.setSource(Qt::MouseEventSynthesizedByQt);
+#endif
             me.setAccepted(d->isPressAndHoldConnected());
             emit pressAndHold(&me);
             if (!me.isAccepted())
@@ -1031,7 +1039,7 @@ void QQuickMouseArea::geometryChange(const QRectF &newGeometry, const QRectF &ol
     Q_D(QQuickMouseArea);
     QQuickItem::geometryChange(newGeometry, oldGeometry);
 
-    if (d->lastScenePos.isNull)
+    if (!d->lastScenePos.isValid())
         d->lastScenePos = mapToScene(d->lastPos);
     else if (newGeometry.x() != oldGeometry.x() || newGeometry.y() != oldGeometry.y())
         d->lastPos = mapFromScene(d->lastScenePos);
@@ -1148,7 +1156,7 @@ bool QQuickMouseArea::hovered() const
     \qmlproperty bool QtQuick::MouseArea::pressed
     This property holds whether any of the \l acceptedButtons are currently pressed.
 */
-bool QQuickMouseArea::pressed() const
+bool QQuickMouseArea::isPressed() const
 {
     Q_D(const QQuickMouseArea);
     return d->pressed;
@@ -1222,6 +1230,10 @@ bool QQuickMouseArea::setPressed(Qt::MouseButton button, bool p, Qt::MouseEventS
 {
     Q_D(QQuickMouseArea);
 
+    // Don't allow entering pressed state while invisible
+    if (p && !d->effectiveVisible)
+        return false;
+
 #if QT_CONFIG(quick_draganddrop)
     bool dragged = d->drag && d->drag->active();
 #else
@@ -1234,7 +1246,9 @@ bool QQuickMouseArea::setPressed(Qt::MouseButton button, bool p, Qt::MouseEventS
     if (wasPressed != p) {
         QQuickMouseEvent &me = d->quickMouseEvent;
         me.reset(d->lastPos.x(), d->lastPos.y(), d->lastButton, d->lastButtons, d->lastModifiers, isclick, d->longPress, d->lastFlags);
+#if QT_DEPRECATED_SINCE(6, 6)
         me.setSource(source);
+#endif
         if (p) {
             d->pressed |= button;
             if (!d->doubleClick)
@@ -1274,6 +1288,7 @@ bool QQuickMouseArea::setPressed(Qt::MouseButton button, bool p, Qt::MouseEventS
 
         return me.isAccepted();
     }
+    Q_UNUSED(source)
     return false;
 }
 
@@ -1318,6 +1333,10 @@ bool QQuickMouseArea::setPressed(Qt::MouseButton button, bool p, Qt::MouseEventS
     \endcode
 
     The default value is \c Qt.ArrowCursor.
+
+    \note If the \c cursorShape property is set to \c undefined, the \c MouseArea will
+    not change the existing shape when entering it.
+
     \sa Qt::CursorShape
 */
 
