@@ -1,5 +1,6 @@
 // Copyright (C) 2021 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquicktreeview_p_p.h"
 
@@ -20,7 +21,7 @@
     A TreeView has a \l model that defines the data to be displayed, and a
     \l delegate that defines how the data should be displayed.
 
-    TreeView inherits \l TableView. This means that even if the model
+    TreeView inherits \l TableView. This means that, despite the model
     has a parent-child tree structure, TreeView is internally using a
     proxy model that converts that structure into a flat table
     model that can be rendered by TableView. Each node in the tree ends up
@@ -29,13 +30,46 @@
     parent-child depth in the model, it will end up looking like a tree, even
     if it's technically still just a flat list of items.
 
-    To allow for maximum flexibility, TreeView itself will not position the delegate
-    items into a tree structure. This burden is placed on the delegate.
-    \l {Qt Quick Controls} offers a ready-made TreeViewDelegate that can be
-    used for this, which has the advantage that it works out-of-the-box and
-    renders a tree which follows the style of the platform where the application runs.
+    \section2 Declare a TreeView
 
-    Even if TreeViewDelegate is customizable, there might be situations
+    TreeView is a data bound control, so it cannot show anything without a
+    data model. You cannot declare tree nodes in QML.
+
+    When you declare a TreeView, you need to specify:
+
+    \list
+      \li \b{A data model}. TreeView can work with data models that derive from
+        \l QAbstractItemModel.
+      \li \b{A delegate}. A delegate is a template that specifies how the tree
+        nodes are displayed in the UI.
+    \endlist
+
+    \qml
+    TreeView {
+        // The model needs to be a QAbstractItemModel
+        model: myTreeModel
+        // You can set a custom delegate or use a built-in TreeViewDelegate
+        delegate: TreeViewDelegate {}
+    }
+    \endqml
+
+    \section2 Creating a data model
+
+    A TreeView only accepts a model that inherits \l QAbstractItemModel.
+
+    For information on how to create and use a custom tree model, see the
+    example: \l {Qt Quick Controls - Table of Contents}.
+
+    \section2 Customize tree nodes
+
+    For better flexibility, TreeView itself doesn't position the delegate items
+    into a tree structure. This burden is placed on the delegate.
+    \l {Qt Quick Controls} offers a ready-made \l TreeViewDelegate that can be
+    used for this, which has the advantage that it works out-of-the-box and
+    renders a tree which follows the style of the platform where the application
+    runs.
+
+    Even though \l TreeViewDelegate is customizable, there might be situations
     where you want to render the tree in a different way, or ensure that
     the delegate ends up as minimal as possible, perhaps for performance reasons.
     Creating your own delegate from scratch is easy, since TreeView offers
@@ -46,8 +80,6 @@
 
     \snippet qml/treeview/qml-customdelegate.qml 0
 
-    More information on how to create and use a custom tree model can be found
-    in the example \l {Qt Quick Controls - Table of Contents}
 
     The properties that are marked as \c required will be filled in by
     TreeView, and are similar to attached properties. By marking them as
@@ -82,6 +114,8 @@
 
     See also \l {Required Properties}.
 
+    \section2 End-user interaction
+
     By default, TreeView \l {toggleExpanded()}{toggles} the expanded state
     of a row when you double tap on it. Since this is in conflict with
     double tapping to edit a cell, TreeView sets \l {TableView::}{editTriggers} to
@@ -90,7 +124,6 @@
     If you change \l {TableView::}{editTriggers} to also contain \c TableView.DoubleTapped,
     toggling the expanded state with a double tap will be disabled.
 
-    \note A TreeView only accepts a model that inherits \l QAbstractItemModel.
 */
 
 /*!
@@ -282,13 +315,9 @@ void QQuickTreeViewPrivate::setModelImpl(const QVariant &newModel)
     Q_Q(QQuickTreeView);
 
     m_assignedModel = newModel;
-    QVariant effectiveModel = m_assignedModel;
-    if (effectiveModel.userType() == qMetaTypeId<QJSValue>())
-        effectiveModel = effectiveModel.value<QJSValue>().toVariant();
-
-    if (effectiveModel.isNull())
+    if (m_assignedModel.isNull())
         m_treeModelToTableModel.setModel(nullptr);
-    else if (const auto qaim = qvariant_cast<QAbstractItemModel*>(effectiveModel))
+    else if (const auto qaim = qvariant_cast<QAbstractItemModel *>(m_assignedModel))
         m_treeModelToTableModel.setModel(qaim);
     else
         qmlWarning(q) << "TreeView only accepts a model of type QAbstractItemModel";
@@ -668,8 +697,10 @@ void QQuickTreeView::keyPressEvent(QKeyEvent *event)
         break;
     }
 
-    if (!event->isAccepted())
+    if (!event->isAccepted()) {
+        event->accept();
         QQuickTableView::keyPressEvent(event);
+    }
 }
 
 QT_END_NAMESPACE
