@@ -11,9 +11,17 @@ QT_BEGIN_NAMESPACE
 QSGSoftwareInternalRectangleNode::QSGSoftwareInternalRectangleNode()
     : m_penWidth(0)
     , m_radius(0)
+    , m_topLeftRadius(0)
+    , m_topRightRadius(0)
+    , m_bottomLeftRadius(0)
+    , m_bottomRightRadius(0)
+    , m_devicePixelRatio(1)
     , m_vertical(true)
     , m_cornerPixmapIsDirty(true)
-    , m_devicePixelRatio(1)
+    , m_isTopLeftRadiusSet(false)
+    , m_isTopRightRadiusSet(false)
+    , m_isBottomLeftRadiusSet(false)
+    , m_isBottomRightRadiusSet(false)
 {
     m_pen.setJoinStyle(Qt::MiterJoin);
     m_pen.setMiterLimit(0);
@@ -164,11 +172,87 @@ void QSGSoftwareInternalRectangleNode::setGradientVertical(bool vertical)
 
 void QSGSoftwareInternalRectangleNode::setRadius(qreal radius)
 {
-    if (m_radius != radius) {
-        m_radius = radius;
-        m_cornerPixmapIsDirty = true;
-        markDirty(DirtyMaterial);
-    }
+    if (m_radius == radius)
+        return;
+    m_radius = radius;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
+}
+
+void QSGSoftwareInternalRectangleNode::setTopLeftRadius(qreal radius)
+{
+    if (m_isTopLeftRadiusSet && m_topLeftRadius == radius)
+        return;
+    m_isTopLeftRadiusSet = true;
+    m_topLeftRadius = radius;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
+}
+
+void QSGSoftwareInternalRectangleNode::setTopRightRadius(qreal radius)
+{
+    if (m_isTopRightRadiusSet && m_topRightRadius == radius)
+        return;
+    m_isTopRightRadiusSet = true;
+    m_topRightRadius = radius;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
+}
+
+void QSGSoftwareInternalRectangleNode::setBottomLeftRadius(qreal radius)
+{
+    if (m_isBottomLeftRadiusSet && m_bottomLeftRadius == radius)
+        return;
+    m_isBottomLeftRadiusSet = true;
+    m_bottomLeftRadius = radius;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
+}
+
+void QSGSoftwareInternalRectangleNode::setBottomRightRadius(qreal radius)
+{
+    if (m_isBottomRightRadiusSet && m_bottomRightRadius == radius)
+        return;
+    m_isBottomRightRadiusSet = true;
+    m_bottomRightRadius = radius;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
+}
+
+void QSGSoftwareInternalRectangleNode::resetTopLeftRadius()
+{
+    if (!m_isTopLeftRadiusSet)
+        return;
+    m_isTopLeftRadiusSet = false;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
+}
+
+void QSGSoftwareInternalRectangleNode::resetTopRightRadius()
+{
+    if (!m_isTopRightRadiusSet)
+        return;
+    m_isTopRightRadiusSet = false;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
+}
+
+void QSGSoftwareInternalRectangleNode::resetBottomLeftRadius()
+{
+    if (!m_isBottomLeftRadiusSet)
+        return;
+    m_isBottomLeftRadiusSet = false;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
+}
+
+void QSGSoftwareInternalRectangleNode::resetBottomRightRadius()
+{
+    if (!m_isBottomRightRadiusSet)
+        return;
+    m_isBottomRightRadiusSet = false;
+    m_cornerPixmapIsDirty = true;
+    markDirty(DirtyMaterial);
 }
 
 void QSGSoftwareInternalRectangleNode::setAligned(bool /*aligned*/)
@@ -185,9 +269,8 @@ void QSGSoftwareInternalRectangleNode::update()
     }
 
     if (!m_stops.isEmpty()) {
-        QLinearGradient gradient(QPoint(0,0), QPoint(m_vertical ? 0 : 1, m_vertical ? 1 : 0));
+        QLinearGradient gradient(QPoint(0,0), QPoint(m_vertical ? 0 : m_rect.width(), m_vertical ? m_rect.height() : 0));
         gradient.setStops(m_stops);
-        gradient.setCoordinateMode(QGradient::ObjectBoundingMode);
         m_brush = QBrush(gradient);
     } else {
         m_brush = QBrush(m_color);
@@ -212,13 +295,21 @@ void QSGSoftwareInternalRectangleNode::paint(QPainter *painter)
         //Rotated rectangles lose the benefits of direct rendering, and have poor rendering
         //quality when using only blits and fills.
 
-        if (m_radius == 0 && m_penWidth == 0) {
+        if (m_radius == 0
+            && m_penWidth == 0
+            && !m_isTopLeftRadiusSet
+            && !m_isTopRightRadiusSet
+            && !m_isBottomLeftRadiusSet
+            && !m_isBottomRightRadiusSet) {
             //Non-Rounded Rects without borders (fall back to drawRect)
             //Most common case
             painter->setPen(Qt::NoPen);
             painter->setBrush(m_brush);
             painter->drawRect(m_rect);
-        } else {
+        } else if (!m_isTopLeftRadiusSet
+                   && !m_isTopRightRadiusSet
+                   && !m_isBottomLeftRadiusSet
+                   && !m_isBottomRightRadiusSet) {
             //Rounded Rects and Rects with Borders
             //Avoids broken behaviors of QPainter::drawRect/roundedRect
             QPixmap pixmap = QPixmap(qRound(m_rect.width() * m_devicePixelRatio), qRound(m_rect.height() * m_devicePixelRatio));
@@ -231,19 +322,44 @@ void QSGSoftwareInternalRectangleNode::paint(QPainter *painter)
             painter->setRenderHint(QPainter::SmoothPixmapTransform, true);
             painter->drawPixmap(m_rect, pixmap);
             painter->setRenderHints(previousRenderHints);
+        } else {
+            // Corners with different radii. Split implementation to avoid
+            // performance regression of the majority of cases
+            QPixmap pixmap = QPixmap(qRound(m_rect.width() * m_devicePixelRatio), qRound(m_rect.height() * m_devicePixelRatio));
+            pixmap.fill(Qt::transparent);
+            pixmap.setDevicePixelRatio(m_devicePixelRatio);
+            QPainter pixmapPainter(&pixmap);
+            // Slow function relying on paths
+            paintRectangleIndividualCorners(&pixmapPainter, QRect(0, 0, m_rect.width(), m_rect.height()));
+
+            QPainter::RenderHints previousRenderHints = painter->renderHints();
+            painter->setRenderHint(QPainter::SmoothPixmapTransform, true);
+            painter->drawPixmap(m_rect, pixmap);
+            painter->setRenderHints(previousRenderHints);
+
         }
 
 
     } else {
         //Paint directly
-        paintRectangle(painter, m_rect);
+        if (!m_isTopLeftRadiusSet
+            && !m_isTopRightRadiusSet
+            && !m_isBottomLeftRadiusSet
+            && !m_isBottomRightRadiusSet) {
+            paintRectangle(painter, m_rect);
+        } else {
+            paintRectangleIndividualCorners(painter, m_rect);
+        }
     }
-
 }
 
 bool QSGSoftwareInternalRectangleNode::isOpaque() const
 {
-    if (m_radius > 0.0f)
+    if (m_radius > 0.0f
+        || m_topLeftRadius > 0.0f
+        || m_topRightRadius > 0.0f
+        || m_bottomRightRadius > 0.0f
+        || m_bottomLeftRadius > 0.0f)
         return false;
     if (m_color.alpha() < 255)
         return false;
@@ -269,7 +385,6 @@ void QSGSoftwareInternalRectangleNode::paintRectangle(QPainter *painter, const Q
 {
     //Radius should never exceeds half of the width or half of the height
     int radius = qFloor(qMin(qMin(rect.width(), rect.height()) * 0.5, m_radius));
-
     QPainter::RenderHints previousRenderHints = painter->renderHints();
     painter->setRenderHint(QPainter::Antialiasing, false);
 
@@ -387,6 +502,79 @@ void QSGSoftwareInternalRectangleNode::paintRectangle(QPainter *painter, const Q
     painter->setRenderHints(previousRenderHints);
 }
 
+void QSGSoftwareInternalRectangleNode::paintRectangleIndividualCorners(QPainter *painter, const QRect &rect)
+{
+    QPainterPath path;
+
+    const float w = m_penWidth;
+
+    // Radius should never exceeds half of the width or half of the height
+    const float radiusTL = qMin(qMin(rect.width(), rect.height()) * 0.5f,
+                                m_isTopLeftRadiusSet ? m_topLeftRadius : m_radius);
+    const float radiusTR = qMin(qMin(rect.width(), rect.height()) * 0.5f,
+                                m_isTopRightRadiusSet ? m_topRightRadius : m_radius);
+    const float radiusBL = qMin(qMin(rect.width(), rect.height()) * 0.5f,
+                                m_isBottomLeftRadiusSet ? m_bottomLeftRadius : m_radius);
+    const float radiusBR = qMin(qMin(rect.width(), rect.height()) * 0.5f,
+                                m_isBottomRightRadiusSet ? m_bottomRightRadius : m_radius);
+
+    const float innerRadiusTL = qMin(qMin(rect.width(), rect.height()) * 0.5f, radiusTL - w);
+    const float innerRadiusTR = qMin(qMin(rect.width(), rect.height()) * 0.5f, radiusTR - w);
+    const float innerRadiusBL = qMin(qMin(rect.width(), rect.height()) * 0.5f, radiusBL - w);
+    const float innerRadiusBR = qMin(qMin(rect.width(), rect.height()) * 0.5f, radiusBR - w);
+
+    QRect rect2 = rect.adjusted(0, 0, 1, 1);
+
+    path.moveTo(rect2.topRight() - QPointF(radiusTR, -w));
+    if (innerRadiusTR > 0.)
+        path.arcTo(QRectF(rect2.topRight() - QPointF(radiusTR + innerRadiusTR, -w), 2. * QSizeF(innerRadiusTR, innerRadiusTR)), 90, -90);
+    else
+        path.lineTo(rect2.topRight() - QPointF(w, -w));
+
+    if (innerRadiusBR > 0.)
+        path.arcTo(QRectF(rect2.bottomRight() - QPointF(radiusBR + innerRadiusBR, radiusBR + innerRadiusBR), 2. * QSizeF(innerRadiusBR, innerRadiusBR)), 0, -90);
+    else
+        path.lineTo(rect2.bottomRight() - QPointF(w, w));
+
+    if (innerRadiusBL > 0.)
+        path.arcTo(QRectF(rect2.bottomLeft() - QPointF(-w, radiusBL + innerRadiusBL), 2. * QSizeF(innerRadiusBL, innerRadiusBL)), -90, -90);
+    else
+        path.lineTo(rect2.bottomLeft() - QPointF(-w, w));
+    if (innerRadiusTL > 0.)
+        path.arcTo(QRectF(rect2.topLeft() + QPointF(w, w), 2. * QSizeF(innerRadiusTL, innerRadiusTL)), -180, -90);
+    else
+        path.lineTo(rect2.topLeft() + QPointF(w, w));
+    path.closeSubpath();
+
+    painter->setPen(Qt::NoPen);
+    painter->setBrush(m_brush);
+    painter->drawPath(path);
+
+    if (w > 0) {
+        path.moveTo(rect2.topRight() - QPointF(radiusTR, 0.));
+        if (radiusTR > 0.)
+            path.arcTo(QRectF(rect2.topRight() - 2. * QPointF(radiusTR, 0.), 2. * QSizeF(radiusTR, radiusTR)), 90, -90);
+        else
+            path.lineTo(rect2.topRight());
+        if (radiusBR > 0.)
+            path.arcTo(QRectF(rect2.bottomRight() - 2. * QPointF(radiusBR, radiusBR), 2. * QSizeF(radiusBR, radiusBR)), 0, -90);
+        else
+            path.lineTo(rect2.bottomRight());
+        if (radiusBL > 0.)
+            path.arcTo(QRectF(rect2.bottomLeft() - 2. * QPointF(0., radiusBL), 2. * QSizeF(radiusBL, radiusBL)), -90, -90);
+        else
+            path.lineTo(rect2.bottomLeft());
+        if (radiusTL > 0.)
+            path.arcTo(QRectF(rect2.topLeft() - 2. * QPointF(0., 0.), 2. * QSizeF(radiusTL, radiusTL)), -180, -90);
+        else
+            path.lineTo(rect2.topLeft());
+        path.closeSubpath();
+
+        painter->setBrush(m_penColor);
+        painter->drawPath(path);
+    }
+}
+
 void QSGSoftwareInternalRectangleNode::generateCornerPixmap()
 {
     //Generate new corner Pixmap
