@@ -1,41 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquicksmoothedanimation_p.h"
 #include "qquicksmoothedanimation_p_p.h"
@@ -88,14 +53,12 @@ QSmoothedAnimation::~QSmoothedAnimation()
     delete delayedStopTimer;
     if (animationTemplate) {
         if (target.object()) {
-            QHash<QQmlProperty, QSmoothedAnimation* >::iterator it =
-                    animationTemplate->activeAnimations.find(target);
-            if (it != animationTemplate->activeAnimations.end() && it.value() == this)
+            auto it = animationTemplate->activeAnimations.constFind(target);
+            if (it != animationTemplate->activeAnimations.cend() && it.value() == this)
                 animationTemplate->activeAnimations.erase(it);
         } else {
             //target is no longer valid, need to search linearly
-            QHash<QQmlProperty, QSmoothedAnimation* >::iterator it;
-            for (it = animationTemplate->activeAnimations.begin(); it != animationTemplate->activeAnimations.end(); ++it) {
+            for (auto it = animationTemplate->activeAnimations.cbegin(); it != animationTemplate->activeAnimations.cend(); ++it) {
                 if (it.value() == this) {
                     animationTemplate->activeAnimations.erase(it);
                     break;
@@ -320,7 +283,7 @@ void QSmoothedAnimation::debugAnimation(QDebug d) const
 
 /*!
     \qmltype SmoothedAnimation
-    \instantiates QQuickSmoothedAnimation
+    \nativetype QQuickSmoothedAnimation
     \inqmlmodule QtQuick
     \ingroup qtquick-transitions-animations
     \inherits NumberAnimation
@@ -387,7 +350,7 @@ QQuickSmoothedAnimationPrivate::~QQuickSmoothedAnimationPrivate()
 
 void QQuickSmoothedAnimationPrivate::updateRunningAnimations()
 {
-    for (QSmoothedAnimation *ease : qAsConst(activeAnimations)) {
+    for (QSmoothedAnimation *ease : std::as_const(activeAnimations)) {
         ease->maximumEasingTime = anim->maximumEasingTime;
         ease->reversingMode = anim->reversingMode;
         ease->velocity = anim->velocity;
@@ -457,11 +420,12 @@ QAbstractAnimationJob* QQuickSmoothedAnimation::transition(QQuickStateActions &a
 
     Possible values are:
 
-    \list
-    \li SmoothedAnimation.Eased (default) - the animation will smoothly decelerate, and then reverse direction
-    \li SmoothedAnimation.Immediate - the animation will immediately begin accelerating in the reverse direction, beginning with a velocity of 0
-    \li SmoothedAnimation.Sync - the property is immediately set to the target value
-    \endlist
+    \value SmoothedAnimation.Eased
+        (default) the animation will smoothly decelerate, and then reverse direction
+    \value SmoothedAnimation.Immediate
+        the animation will immediately begin accelerating in the reverse direction, beginning with a velocity of 0
+    \value SmoothedAnimation.Sync
+        the property is immediately set to the target value
 */
 QQuickSmoothedAnimation::ReversingMode QQuickSmoothedAnimation::reversingMode() const
 {
