@@ -1,5 +1,6 @@
 // Copyright (C) 2016 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquicklinearlayout_p.h"
 #include "qquickgridlayoutengine_p.h"
@@ -451,7 +452,8 @@ void QQuickGridLayoutBase::rearrange(const QSizeF &size)
     if (d->m_recurRearrangeCounter++ == 2) {
         // allow a recursive depth of two in order to respond to height-for-width
         // (e.g QQuickText changes implicitHeight when its width gets changed)
-        qWarning() << "Qt Quick Layouts: Detected recursive rearrange. Aborting after two iterations.";
+        qmlWarning(this)
+                << "Qt Quick Layouts: Detected recursive rearrange. Aborting after two iterations.";
         return;
     }
 
