@@ -115,6 +115,9 @@ QQuickShapeGenericRenderer::~QQuickShapeGenericRenderer()
 
 void QQuickShapeGenericRenderer::beginSync(int totalCount, bool *countChanged)
 {
+    for (int i = totalCount; i < m_sp.size(); i++) // Handle removal of paths
+        setFillTextureProvider(i, nullptr); // deref window
+
     if (m_sp.size() != totalCount) {
         m_sp.resize(totalCount);
         m_accDirty |= DirtyList;
@@ -126,11 +129,6 @@ void QQuickShapeGenericRenderer::beginSync(int totalCount, bool *countChanged)
         d.syncDirty = 0;
 }
 
-void QQuickShapeGenericRenderer::setPath(int index, const QQuickPath *path)
-{
-    setPath(index, path ? path->path() : QPainterPath());
-}
-
 void QQuickShapeGenericRenderer::setPath(int index, const QPainterPath &path, QQuickShapePath::PathHints)
 {
     ShapePathData &d(m_sp[index]);
@@ -599,6 +597,15 @@ void QQuickShapeGenericRenderer::updateNode()
         *nodePtr = nullptr;
     }
 
+    if (m_sp.isEmpty()) {
+        delete m_rootNode->m_fillNode;
+        m_rootNode->m_fillNode = nullptr;
+        delete m_rootNode->m_strokeNode;
+        m_rootNode->m_strokeNode = nullptr;
+        delete m_rootNode->m_next;
+        m_rootNode->m_next = nullptr;
+    }
+
     m_accDirty = 0;
 }
 
