@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickdial_p.h"
 #include "qquickdeferredexecute_p_p.h"
@@ -7,6 +8,7 @@
 #include <QtCore/qmath.h>
 #include <QtQuick/private/qquickflickable_p.h>
 #include <QtQuickTemplates2/private/qquickcontrol_p_p.h>
+#include <QtQuickTemplates2/private/qtquicktemplates2math_p.h>
 
 #include <cmath>
 
@@ -342,12 +344,6 @@ void QQuickDialPrivate::executeHandle(bool complete)
         quickCompleteDeferred(q, handleName(), handle);
 }
 
-template<typename ...Real>
-static bool areRepresentableAsInteger(Real... numbers) {
-    auto check = [](qreal number) -> bool { return std::nearbyint(number) == number; };
-    return (... && check(numbers));
-}
-
 void QQuickDialPrivate::updateAllValuesAreInteger()
 {
     allValuesAreInteger = areRepresentableAsInteger(to, from, stepSize) && stepSize != 0.0;
