@@ -1,18 +1,22 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickdialog_p.h"
 #include "qquickdialog_p_p.h"
 #include "qquickdialogbuttonbox_p.h"
 #include "qquickabstractbutton_p.h"
 #include "qquickpopupitem_p_p.h"
+#include "qquickpopupwindow_p_p.h"
+#include <qpa/qplatformintegration.h>
+#include <private/qguiapplication_p.h>
 
 QT_BEGIN_NAMESPACE
 
 /*!
     \qmltype Dialog
     \inherits Popup
-//!     \instantiates QQuickDialog
+//!     \nativetype QQuickDialog
     \inqmlmodule QtQuick.Controls
     \ingroup qtquickcontrols-dialogs
     \ingroup qtquickcontrols-popups
@@ -23,7 +27,7 @@ QT_BEGIN_NAMESPACE
     with the user. Similarly to \l ApplicationWindow and \l Page, Dialog is organized
     into three sections: \l header, \l {Popup::}{contentItem}, and \l footer.
 
-    \image qtquickcontrols-page-wireframe.png
+    \image qtquickcontrols-page-wireframe.webp
 
     The \l {Popup::}{padding} properties only affect the contentItem. Use the
     \l {Popup::}{spacing} property to affect the space between header,
@@ -155,6 +159,7 @@ void QQuickDialogPrivate::handleClick(QQuickAbstractButton *button)
         break;
     case QPlatformDialogHelper::DestructiveRole:
         emit q->discarded();
+        q->close();
         break;
     case QPlatformDialogHelper::HelpRole:
         emit q->helpRequested();
@@ -164,6 +169,11 @@ void QQuickDialogPrivate::handleClick(QQuickAbstractButton *button)
     }
 }
 
+Qt::WindowFlags QQuickDialogPrivate::popupWindowType() const
+{
+    return Qt::Dialog;
+}
+
 QQuickDialog::QQuickDialog(QObject *parent)
     : QQuickDialog(*(new QQuickDialogPrivate), parent)
 {
@@ -176,8 +186,6 @@ QQuickDialog::QQuickDialog(QQuickDialogPrivate &dd, QObject *parent)
 
     // Dialogs should get active focus when opened so that e.g. Cancel closes them.
     setFocus(true);
-
-    QObject::connect(d->popupItem, &QQuickPopupItem::titleChanged, this, &QQuickDialog::titleChanged);
     QObject::connect(d->popupItem, &QQuickPopupItem::headerChanged, this, &QQuickDialog::headerChanged);
     QObject::connect(d->popupItem, &QQuickPopupItem::footerChanged, this, &QQuickDialog::footerChanged);
     QObject::connect(d->popupItem, &QQuickPopupItem::implicitHeaderWidthChanged, this, &QQuickDialog::implicitHeaderWidthChanged);
@@ -189,7 +197,6 @@ QQuickDialog::QQuickDialog(QQuickDialogPrivate &dd, QObject *parent)
 QQuickDialog::~QQuickDialog()
 {
     Q_D(QQuickDialog);
-    QObject::disconnect(d->popupItem, &QQuickPopupItem::titleChanged, this, &QQuickDialog::titleChanged);
     QObject::disconnect(d->popupItem, &QQuickPopupItem::headerChanged, this, &QQuickDialog::headerChanged);
     QObject::disconnect(d->popupItem, &QQuickPopupItem::footerChanged, this, &QQuickDialog::footerChanged);
     QObject::disconnect(d->popupItem, &QQuickPopupItem::implicitHeaderWidthChanged, this, &QQuickDialog::implicitHeaderWidthChanged);
@@ -218,13 +225,22 @@ QQuickDialog::~QQuickDialog()
 QString QQuickDialog::title() const
 {
     Q_D(const QQuickDialog);
-    return d->popupItem->title();
+    return d->title;
 }
 
 void QQuickDialog::setTitle(const QString &title)
 {
     Q_D(QQuickDialog);
-    d->popupItem->setTitle(title);
+    if (d->title == title)
+        return;
+    d->title = title;
+
+    if (d->popupWindow)
+        d->popupWindow->setTitle(title);
+    else
+        d->popupItem->setTitle(title);
+
+    emit titleChanged();
 }
 
 /*!
@@ -488,6 +504,12 @@ qreal QQuickDialog::implicitFooterHeight() const
     return d->popupItem->implicitFooterHeight();
 }
 
+void QQuickDialog::setOpacity(qreal opacity)
+{
+    Q_D(QQuickDialog);
+    QQuickPopup::setOpacity(d->popupWindow ? qreal(!qFuzzyIsNull(opacity)) : opacity);
+}
+
 /*!
     \qmlmethod void QtQuick.Controls::Dialog::accept()
 
