@@ -1,5 +1,6 @@
 // Copyright (C) 2025 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquicksearchfield_p.h"
 #include "qquickcontrol_p_p.h"
@@ -57,11 +58,24 @@ QT_BEGIN_NAMESPACE
         ListElement { name: "WaterMelon"; color: "pink" }
     }
 
-    QSortFilterProxyModel {
+    SortFilterProxyModel {
         id: fruitFilter
-        sourceModel: fruitModel
-        filterRegularExpression: RegExp(fruitSearch.text, "i")
-        filterRole: 0 // needs to be set explicitly
+        model: fruitModel
+        sorters: [
+            RoleSorter {
+                roleName: "name"
+            }
+        ]
+        filters: [
+            FunctionFilter {
+                component CustomData: QtObject { property string name }
+                property var regExp: new RegExp(fruitSearch.text, "i")
+                onRegExpChanged: invalidate()
+                function filter(data: CustomData): bool {
+                    return regExp.test(data.name);
+                }
+            }
+        ]
     }
 
     SearchField {
@@ -363,9 +377,6 @@ void QQuickSearchFieldPrivate::increaseCurrentIndex()
     if (isPopupVisible()) {
         if (highlightedIndex < q->suggestionCount() - 1)
             setHighlightedIndex(highlightedIndex + 1, Highlight);
-    } else {
-        if (currentIndex < q->suggestionCount() - 1)
-            setCurrentItemAtIndex(currentIndex + 1, Activate);
     }
 }
 
@@ -374,9 +385,6 @@ void QQuickSearchFieldPrivate::decreaseCurrentIndex()
     if (isPopupVisible()) {
         if (highlightedIndex > 0)
             setHighlightedIndex(highlightedIndex - 1, Highlight);
-    } else {
-        if (currentIndex > 0)
-            setCurrentItemAtIndex(currentIndex - 1, Activate);
     }
 }
 
@@ -407,7 +415,17 @@ void QQuickSearchFieldPrivate::setCurrentItemAtIndex(int index, Activation activ
 
 void QQuickSearchFieldPrivate::updateHighlightedIndex()
 {
-    setHighlightedIndex(popup->isVisible() ? currentIndex : -1, NoHighlight);
+    Q_Q(QQuickSearchField);
+    int index = -1;
+
+    if (isPopupVisible()) {
+        if (currentIndex >= 0)
+            index = currentIndex;
+        else if (q->suggestionCount() > 0)
+            index = 0; // auto-highlight first suggestion
+    }
+
+    setHighlightedIndex(index, NoHighlight);
 }
 
 void QQuickSearchFieldPrivate::setHighlightedIndex(int index, Highlighting highlight)
@@ -489,6 +507,9 @@ void QQuickSearchFieldPrivate::updateText()
         q->setText(textInput);
         emit q->textEdited();
 
+        setCurrentIndex(-1);
+        updateHighlightedIndex();
+
         if (live)
             emit q->searchTriggered();
     }
@@ -707,9 +728,6 @@ void QQuickSearchField::setSuggestionModel(const QVariant &model)
     d->suggestionModel = suggestionModel;
     d->createDelegateModel();
     emit suggestionCountChanged();
-    if (isComponentComplete()) {
-        setCurrentIndex(suggestionCount() > 0 ? 0 : -1);
-    }
     emit suggestionModelChanged();
 }
 
@@ -1101,15 +1119,11 @@ void QQuickSearchField::keyPressEvent(QKeyEvent *event)
         case Qt::Key_Home:
             if (d->isPopupVisible())
                 d->setHighlightedIndex(0, Highlight);
-            else
-                d->setCurrentItemAtIndex(0, Activate);
             event->accept();
             break;
         case Qt::Key_End:
             if (d->isPopupVisible())
                 d->setHighlightedIndex(suggestionCount() - 1, Highlight);
-            else
-                d->setCurrentItemAtIndex(suggestionCount() - 1, Activate);
             event->accept();
             break;
         default:
@@ -1143,11 +1157,6 @@ void QQuickSearchField::componentComplete()
 
     if (d->delegateModel && d->ownModel)
         static_cast<QQmlDelegateModel *>(d->delegateModel)->componentComplete();
-
-    if (suggestionCount() > 0) {
-        if (!d->hasCurrentIndex && d->currentIndex == -1)
-            setCurrentIndex(0);
-    }
 }
 
 void QQuickSearchField::contentItemChange(QQuickItem *newItem, QQuickItem *oldItem)
