@@ -1,5 +1,6 @@
 // Copyright (C) 2017 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
+// Qt-Security score:significant reason:default
 
 #include "qquickspinbox_p.h"
 
@@ -23,7 +24,7 @@ static const int AUTO_REPEAT_INTERVAL = 100;
 //!     \nativetype QQuickSpinBox
     \inqmlmodule QtQuick.Controls
     \since 5.7
-    \ingroup input
+    \ingroup qtquickcontrols-input
     \ingroup qtquickcontrols-focusscopes
     \brief Allows the user to select from a set of preset values.
 
@@ -239,7 +240,7 @@ void QQuickSpinBoxPrivate::contentItemTextChanged()
         return;
     QString text = inputTextItem->text();
 #if QT_CONFIG(validator)
-    if (validator)
+    if (validator && live)
         validator->fixup(text);
 #endif
 
@@ -343,12 +344,17 @@ bool QQuickSpinBoxPrivate::handleMove(const QPointF &point, ulong timestamp)
 {
     Q_Q(QQuickSpinBox);
     QQuickControlPrivate::handleMove(point, timestamp);
-    QQuickItem *ui = up->indicator();
-    QQuickItem *di = down->indicator();
-    up->setHovered(ui && ui->isEnabled() && ui->contains(ui->mapFromItem(q, point)));
-    up->setPressed(up->isHovered());
-    down->setHovered(di && di->isEnabled() && di->contains(di->mapFromItem(q, point)));
-    down->setPressed(down->isHovered());
+    QQuickItem *upIndicator = up->indicator();
+    const bool upIndicatorContainsPoint = upIndicator && upIndicator->isEnabled()
+        && upIndicator->contains(upIndicator->mapFromItem(q, point));
+    up->setHovered(touchId == -1 && upIndicatorContainsPoint);
+    up->setPressed(upIndicatorContainsPoint);
+
+    QQuickItem *downIndicator = down->indicator();
+    const bool downIndicatorContainsPoint = downIndicator && downIndicator->isEnabled()
+        && downIndicator->contains(downIndicator->mapFromItem(q, point));
+    down->setHovered(touchId == -1 && downIndicatorContainsPoint);
+    down->setPressed(downIndicatorContainsPoint);
 
     bool pressed = up->isPressed() || down->isPressed();
     q->setAccessibleProperty("pressed", pressed);
@@ -476,6 +482,9 @@ QQuickSpinBox::QQuickSpinBox(QQuickItem *parent)
 #if QT_CONFIG(cursor)
     setCursor(Qt::ArrowCursor);
 #endif
+#if QT_CONFIG(quicktemplates2_multitouch)
+    setAcceptTouchEvents(true);
+#endif
 }
 
 QQuickSpinBox::~QQuickSpinBox()
