@@ -1341,12 +1341,12 @@ void QQuickWidget::continueExecute()
     if (d->source.isEmpty())
         d->source = d->component->url();
 
-    if (d->setRootObject(obj.get()))
+    d->setRootObject(obj.get());
+    if (d->root)
         Q_UNUSED(obj.release());
     emit statusChanged(status());
 }
 
-
 /*!
   \internal
 */
