@@ -2,10 +2,12 @@
 // This code is governed by the BSD license found in the LICENSE file.
 
 /*---
-es5id: 15.12.1.1-0-1
-description: The JSON lexical grammar treats whitespace as a token seperator
+es5id: 15.12.1.1-0-4
+description: >
+    <NBSP> is not valid JSON whitespace as specified by the production
+    JSONWhitespace.
 ---*/
 
 assert.throws(SyntaxError, function() {
-  JSON.parse('12\t\r\n 34'); // should produce a syntax error as whitespace results in two tokens
+  JSON.parse('\u00a01234'); // should produce a syntax error 
 });
