@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <private/qqmljsengine_p.h>
 #include <private/qqmljsparser_p.h>
@@ -59,6 +34,10 @@ private slots:
     void codeLocationsWithContinuationStringLiteral_data();
     void noSubstitutionTemplateLiteral();
     void templateLiteral();
+    void numericSeparator_data();
+    void numericSeparator();
+    void invalidNumericSeparator_data();
+    void invalidNumericSeparator();
     void leadingSemicolonInClass();
     void templatedReadonlyProperty();
     void qmlImportInJS();
@@ -74,6 +53,8 @@ private slots:
     void invalidImportVersion_data();
     void invalidImportVersion();
 
+    void trailingCommaInUiArray();
+
 private:
     QStringList excludedDirs;
 
@@ -98,8 +79,8 @@ public:
     {
         if (! nodeStack.isEmpty()) {
             AST::Node *parent = nodeStack.last();
-            const quint32 parentBegin = parent->firstSourceLocation().begin();
-            const quint32 parentEnd = parent->lastSourceLocation().end();
+            const qsizetype parentBegin = parent->firstSourceLocation().begin();
+            const qsizetype parentEnd = parent->lastSourceLocation().end();
 
             if (node->firstSourceLocation().begin() < parentBegin)
                 qDebug() << "first source loc failed: node:" << node->kind << "at" << node->firstSourceLocation().startLine << "/" << node->firstSourceLocation().startColumn
@@ -216,7 +197,7 @@ public:
             ++startLine;
         }
         QCOMPARE(expected, found);
-        SourceLocation combined(first.offset, last.end() - first.begin(),
+        SourceLocation combined(first.offset, quint32(last.end() - first.begin()),
                                 first.startLine, first.startColumn);
         SourceLocation cStart = combined.startZeroLengthLocation();
         SourceLocation cEnd = combined.endZeroLengthLocation(m_codeStr);
@@ -230,7 +211,7 @@ public:
         int lastNewline = found.lastIndexOf(QLatin1Char('\n'));
         if (lastNewline < 0)
             lastNewline = -int(combined.startColumn);
-        QCOMPARE(cEnd.startColumn, found.length() - lastNewline);
+        QCOMPARE(cEnd.startColumn, found.size() - lastNewline);
     }
 private:
     QString m_codeStr;
@@ -260,7 +241,7 @@ void tst_qqmlparser::initTestCase()
 
 QStringList tst_qqmlparser::findFiles(const QDir &d)
 {
-    for (int ii = 0; ii < excludedDirs.count(); ++ii) {
+    for (int ii = 0; ii < excludedDirs.size(); ++ii) {
         QString s = excludedDirs.at(ii);
         if (d.absolutePath().endsWith(s))
             return QStringList();
@@ -268,15 +249,13 @@ QStringList tst_qqmlparser::findFiles(const QDir &d)
 
     QStringList rv;
 
-    QStringList files = d.entryList(QStringList() << QLatin1String("*.qml") << QLatin1String("*.js"),
-                                    QDir::Files);
-    foreach (const QString &file, files) {
+    const QStringList files = d.entryList(
+            QStringList() << QLatin1String("*.qml") << QLatin1String("*.js"), QDir::Files);
+    for (const QString &file : files)
         rv << d.absoluteFilePath(file);
-    }
 
-    QStringList dirs = d.entryList(QDir::Dirs | QDir::NoDotAndDotDot |
-                                   QDir::NoSymLinks);
-    foreach (const QString &dir, dirs) {
+    const QStringList dirs = d.entryList(QDir::Dirs | QDir::NoDotAndDotDot | QDir::NoSymLinks);
+    for (const QString &dir : dirs) {
         QDir sub = d;
         sub.cd(dir);
         rv << findFiles(sub);
@@ -302,7 +281,7 @@ void tst_qqmlparser::qmlParser_data()
     files << findFiles(QDir(examples));
     files << findFiles(QDir(tests));
 
-    foreach (const QString &file, files)
+    for (const QString &file : std::as_const(files))
         QTest::newRow(qPrintable(file)) << file;
 }
 #endif
@@ -361,7 +340,7 @@ void tst_qqmlparser::stringLiteral()
     auto *literal = QQmlJS::AST::cast<QQmlJS::AST::StringLiteral *>(expression);
     QVERIFY(literal);
     QCOMPARE(literal->value, u"hello string");
-    QCOMPARE(literal->firstSourceLocation().begin(), 0u);
+    QCOMPARE(literal->firstSourceLocation().begin(), 0);
     QCOMPARE(literal->lastSourceLocation().end(), quint32(code.size()));
 
     // test for correct handling escape sequences inside strings
@@ -380,7 +359,7 @@ void tst_qqmlparser::stringLiteral()
     literal = QQmlJS::AST::cast<QQmlJS::AST::StringLiteral *>(binaryExpression->left);
     QVERIFY(literal);
     QCOMPARE(literal->value, u"hello\n\tstring");
-    QCOMPARE(literal->firstSourceLocation().begin(), 0u);
+    QCOMPARE(literal->firstSourceLocation().begin(), 0);
     QCOMPARE(literal->firstSourceLocation().startLine, 1u);
     QCOMPARE(literal->lastSourceLocation().end(), quint32(leftCode.size()));
 
@@ -388,7 +367,7 @@ void tst_qqmlparser::stringLiteral()
     literal = QQmlJS::AST::cast<QQmlJS::AST::StringLiteral *>(binaryExpression->right);
     QVERIFY(literal);
     QCOMPARE(literal->value, u"\nbye");
-    quint32 offset = quint32(leftCode.size() + plusCode.size());
+    qsizetype offset = leftCode.size() + plusCode.size();
     QCOMPARE(literal->firstSourceLocation().begin(), offset);
     QCOMPARE(literal->firstSourceLocation().startLine, 1u);
     QCOMPARE(literal->lastSourceLocation().end(), quint32(code.size()));
@@ -406,14 +385,14 @@ void tst_qqmlparser::stringLiteral()
     literal = QQmlJS::AST::cast<QQmlJS::AST::StringLiteral *>(binaryExpression->left);
     QVERIFY(literal);
     QCOMPARE(literal->value, u"\nhello\nbye");
-    QCOMPARE(literal->firstSourceLocation().begin(), 0u);
+    QCOMPARE(literal->firstSourceLocation().begin(), 0);
     QCOMPARE(literal->firstSourceLocation().startLine, 1u);
     QCOMPARE(literal->lastSourceLocation().end(), leftCode.size());
 
     literal = QQmlJS::AST::cast<QQmlJS::AST::StringLiteral *>(binaryExpression->right);
     QVERIFY(literal);
     QCOMPARE(literal->value, u"\nbye");
-    offset = quint32(leftCode.size() + plusCode.size());
+    offset = leftCode.size() + plusCode.size();
     QCOMPARE(literal->firstSourceLocation().begin(), offset);
     QCOMPARE(literal->lastSourceLocation().startLine, 3u);
     QCOMPARE(literal->lastSourceLocation().end(), code.size());
@@ -518,11 +497,79 @@ void tst_qqmlparser::templateLiteral()
     auto *templateLiteral = QQmlJS::AST::cast<QQmlJS::AST::TemplateLiteral *>(expression);
     QVERIFY(templateLiteral);
 
-    QCOMPARE(templateLiteral->firstSourceLocation().begin(), 0u);
+    QCOMPARE(templateLiteral->firstSourceLocation().begin(), 0);
     auto *e = templateLiteral->expression;
     QVERIFY(e);
 }
 
+void tst_qqmlparser::numericSeparator_data() {
+    QTest::addColumn<QString>("code");
+    QTest::addColumn<double>("expected_value");
+
+    QTest::newRow("Separator in decimal literal") << "1_000_000_000" << 1000000000.0;
+    QTest::newRow("Separator in fractional part") << "1000.22_33" << 1000.2233;
+    QTest::newRow("Separator in exponent part") << "1e1_0_0" << std::pow(10, 100);
+    QTest::newRow("Separator in positive exponent part") << "1e+1_0_0" << 1e100;
+    QTest::newRow("Separator in negative exponent part") << "1e-1_0_0" << 1e-100;
+    QTest::newRow("Separator in binary literal with b prefix") << "0b1010_0001_1000_0101" << static_cast<double>(0b1010000110000101);
+    QTest::newRow("Separator in binary literal with B prefix") << "0B01_10_01_10" << static_cast<double>(0b01100110);
+    QTest::newRow("Separator in octal literal with o prefix") << "0o1234_5670" << static_cast<double>(012345670);
+    QTest::newRow("Separator in octal literal with O prefix") << "0O7777_0000" << static_cast<double>(077770000);
+    QTest::newRow("Separator in hex literal with x prefix") << "0xA0_B0_C0" << static_cast<double>(0xA0B0C0);
+    QTest::newRow("Separator in hex literal with X prefix") << "0X1000_AAAA" << static_cast<double>(0x1000AAAA);
+}
+
+void tst_qqmlparser::numericSeparator() {
+    using namespace QQmlJS;
+
+    QFETCH(QString, code);
+    QFETCH(double, expected_value);
+
+    QQmlJS::Engine engine;
+
+    QQmlJS::Lexer lexer(&engine);
+    lexer.setCode(code, 1);
+
+    QQmlJS::Parser parser(&engine);
+    QVERIFY(parser.parseExpression());
+
+    AST::ExpressionNode *expression = parser.expression();
+    QVERIFY(expression);
+
+    auto *literal = QQmlJS::AST::cast<QQmlJS::AST::NumericLiteral *>(expression);
+    QVERIFY(literal);
+
+    QCOMPARE(literal->value, expected_value);
+    QCOMPARE(literal->firstSourceLocation().begin(), 0);
+    QCOMPARE(literal->lastSourceLocation().end(), quint32(code.size()));
+}
+
+void tst_qqmlparser::invalidNumericSeparator_data() {
+    QTest::addColumn<QString>("code");
+    QTest::addColumn<QString>("error");
+
+    QTest::newRow("Trailing numeric separator") << "1_" << "A trailing numeric separator is not allowed in numeric literals";
+    QTest::newRow("Multiple numeric separators") << "1__2" << "There can be at most one numeric separator between digits";
+}
+
+void tst_qqmlparser::invalidNumericSeparator() {
+    using namespace QQmlJS;
+
+    QFETCH(QString, code);
+    QFETCH(QString, error);
+
+    QQmlJS::Engine engine;
+
+    QQmlJS::Lexer lexer(&engine);
+    lexer.setCode(code, 1);
+
+    QQmlJS::Parser parser(&engine);
+    QVERIFY(!parser.parseExpression());
+
+    QVERIFY(lexer.errorCode() != Lexer::NoError);
+    QCOMPARE(lexer.errorMessage(), error);
+}
+
 void tst_qqmlparser::leadingSemicolonInClass()
 {
     QQmlJS::Engine engine;
@@ -575,7 +622,7 @@ void tst_qqmlparser::typeAnnotations_data()
     QStringList files;
     files << findFiles(QDir(tests));
 
-    for (const QString &file: qAsConst(files))
+    for (const QString &file: std::as_const(files))
         QTest::newRow(qPrintable(file)) << file;
 }
 
@@ -615,7 +662,7 @@ void tst_qqmlparser::disallowedTypeAnnotations_data()
     QStringList files;
     files << findFiles(QDir(tests));
 
-    for (const QString &file: qAsConst(files))
+    for (const QString &file: std::as_const(files))
         QTest::newRow(qPrintable(file)) << file;
 }
 
@@ -704,9 +751,9 @@ void tst_qqmlparser::annotations_data()
     QStringList refFiles;
     refFiles << findFiles(QDir(compare));
 
-    for (const QString &file: qAsConst(files)) {
+    for (const QString &file: std::as_const(files)) {
         auto fileNameStart = file.lastIndexOf(QDir::separator());
-        auto fileName = QStringView(file).mid(fileNameStart, file.length()-fileNameStart);
+        auto fileName = QStringView(file).mid(fileNameStart, file.size()-fileNameStart);
         auto ref=std::find_if(refFiles.constBegin(),refFiles.constEnd(), [fileName](const QString &s){ return s.endsWith(fileName); });
         if (ref != refFiles.constEnd())
             QTest::newRow(qPrintable(file)) << file << *ref;
@@ -794,6 +841,18 @@ void tst_qqmlparser::invalidImportVersion()
     QVERIFY(regexp.match(parser.errorMessage()).hasMatch());
 }
 
+void tst_qqmlparser::trailingCommaInUiArray()
+{
+    QFile file(testFile("trailingComma.qml"));
+    QVERIFY(file.open(QIODevice::ReadOnly));
+
+    QQmlJS::Engine engine;
+    QQmlJS::Lexer lexer(&engine);
+    lexer.setCode(file.readAll(), 1);
+    QQmlJS::Parser parser(&engine);
+    QVERIFY(parser.parse());
+}
+
 QTEST_MAIN(tst_qqmlparser)
 
 #include "tst_qqmlparser.moc"
