@@ -178,6 +178,7 @@ void tst_qqmlqt::rgba()
     QCOMPARE(qvariant_cast<QColor>(object->property("test4")), QColor());
     QCOMPARE(qvariant_cast<QColor>(object->property("test5")), QColor::fromRgbF(1, 1, 1, 1));
     QCOMPARE(qvariant_cast<QColor>(object->property("test6")), QColor::fromRgbF(0, 0, 0, 0));
+    QCOMPARE(qvariant_cast<QColor>(object->property("test7")), QColor::fromRgbF(0, 0, 0, 0));
 }
 
 void tst_qqmlqt::hsla()
@@ -201,6 +202,7 @@ void tst_qqmlqt::hsla()
     QColor test7 = qvariant_cast<QColor>(object->property("test7"));
     QCOMPARE(test7, QColor::fromHslF(-1, 0, 0.5, 1));
     QCOMPARE(test7.hslHue(), -1.0f);
+    QCOMPARE(qvariant_cast<QColor>(object->property("test8")), QColor::fromHslF(0, 0, 0, 0));
 }
 
 void tst_qqmlqt::hsva()
@@ -224,6 +226,7 @@ void tst_qqmlqt::hsva()
     QColor test7 = qvariant_cast<QColor>(object->property("test7"));
     QCOMPARE(test7, QColor::fromHsvF(-1, 0, 0.5, 1));
     QCOMPARE(test7.hsvHue(), -1.0f);
+    QCOMPARE(qvariant_cast<QColor>(object->property("test8")), QColor::fromHsvF(0, 0, 0, 0));
 }
 
 void tst_qqmlqt::colorEqual()
@@ -734,7 +737,7 @@ void tst_qqmlqt::createQmlObject()
     QString warning4 = component.url().toString()+ ":9: TypeError: Passing incompatible arguments to C++ functions from JavaScript is not allowed.";
     QString warning5 = component.url().toString()+ ":8: Error: Too many arguments";
     QString warning6 = "RunTimeError:  Qt.createQmlObject(): failed to create object: \n    " + testFileUrl("inline").toString() + ":3:16: Cannot assign object type QObject with no default method";
-    QString warning7 = "Could not convert argument 1 at";
+    QString warning7 = "Could not convert argument 1 from 0 to QObject*";
     QString warning8 = "expression for noParent@";
     QTest::ignoreMessage(QtWarningMsg, QRegularExpression(warning1));
     QTest::ignoreMessage(QtWarningMsg, qPrintable(warning2));
@@ -827,11 +830,11 @@ void tst_qqmlqt::dateTimeFormatting()
         << "formatting.qml:36: Error: Unable to determine callable overload"
         << "formatting.qml:39: Error: Unable to determine callable overload"
         << "formatting.qml:42: Error: Unable to determine callable overload"
-        << "Could not convert argument 1 at"
+        << "Could not convert argument 1 from \\[object Object\\] to QLocale"
         << "expression for err_date2@"
-        << "Could not convert argument 1 at"
+        << "Could not convert argument 1 from \\[object Object\\] to QLocale"
         << "expression for err_time2@"
-        << "Could not convert argument 1 at"
+        << "Could not convert argument 1 from \\[object Object\\] to QLocale"
         << "expression for err_dateTime2@";
 
     for (const QString &warning : std::as_const(warnings))
@@ -912,11 +915,11 @@ void tst_qqmlqt::dateTimeFormattingVariants()
     warnings << "formatting.qml:36: Error: Unable to determine callable overload."
              << "formatting.qml:39: Error: Unable to determine callable overload."
              << "formatting.qml:42: Error: Unable to determine callable overload."
-             << "Could not convert argument 1 at"
+             << "Could not convert argument 1 from \\[object Object\\] to QLocale"
              << "expression for err_date2@"
-             << "Could not convert argument 1 at"
+             << "Could not convert argument 1 from \\[object Object\\] to QLocale"
              << "expression for err_time2@"
-             << "Could not convert argument 1 at"
+             << "Could not convert argument 1 from \\[object Object\\] to QLocale"
              << "expression for err_dateTime2@";
 
     for (const QString &warning : std::as_const(warnings))
@@ -1115,7 +1118,7 @@ void tst_qqmlqt::dateTimeFormattingWithLocale()
     auto dateString = o->property("dateString").toString();
     QCOMPARE(dateString, QLocale("de_DE").toString(date, QLocale::ShortFormat));
 
-    QTest::ignoreMessage(QtWarningMsg, QRegularExpression("Could not convert argument 1 at"));
+    QTest::ignoreMessage(QtWarningMsg, QRegularExpression("Could not convert argument 1 from null to QLocale"));
     QTest::ignoreMessage(QtWarningMsg, QRegularExpression("invalidUsage@"));
     QTest::ignoreMessage(
                 QtWarningMsg,
