@@ -1304,9 +1304,9 @@ void tst_qmlls_modules::warnings_data()
             << u"warnings/InvalidImport.qml"_s
             << ExpectedWarnings{ {
                        u"Warnings occurred while importing module \"foobar\": [import]"_s,
-                       u"Failed to import foobar. Are your import paths set up properly? Did you build your project? If yes, did you set the \"QT_QML_GENERATE_QMLLS_INI\" CMake variable on your project to \"ON\"? [import]"_s,
+                       u"Failed to import foobar. Are your import paths set up properly? Did you build your project? [import]"_s,
                        u"Warnings occurred while importing module \"foobaz\": [import]"_s,
-                       u"Failed to import foobaz. Are your import paths set up properly? Did you build your project? If yes, did you set the \"QT_QML_GENERATE_QMLLS_INI\" CMake variable on your project to \"ON\"? [import]"_s,
+                       u"Failed to import foobaz. Are your import paths set up properly? Did you build your project? [import]"_s,
                } };
 
     QTest::addRow("WithoutQmllsBuildIni")
@@ -1314,8 +1314,7 @@ void tst_qmlls_modules::warnings_data()
             << ExpectedWarnings{
                    { "Warnings occurred while importing module \"MyModule\": [import]"_L1,
                      "Failed to import MyModule. Are your import paths set up properly? Did you "
-                     "build your project? If yes, did you set the \"QT_QML_GENERATE_QMLLS_INI\" "
-                     "CMake variable on your project to \"ON\"? [import]"_L1,
+                     "build your project? [import]"_L1,
                      "MyComponent was not found. Did you add all imports and dependencies?: Did "
                      "you mean \"Component\"? [import]"_L1 }
                };
@@ -1791,8 +1790,8 @@ void tst_qmlls_modules::semanticHighlightingFull()
 {
     QFETCH(QString, filePath);
     const auto item = fileObject(testFile(filePath));
-    Highlights highlights;
-    const auto expectedData = HighlightingUtils::collectTokens(item, std::nullopt);
+    QmlHighlighting::HighlightsContainer highlights;
+    const auto expectedData = QmlHighlighting::Utils::collectTokens(item, std::nullopt);
 
     const auto uri = openFile(filePath);
     QVERIFY(uri);
@@ -1834,12 +1833,13 @@ void tst_qmlls_modules::semanticHighlightingRange()
     QFETCH(QLspSpecification::Range, range);
 
     const auto item = fileObject(testFile(filePath));
-    Highlights highlights;
+    QmlHighlighting::HighlightsContainer highlights;
     const auto qmlFile = item.as<QQmlJS::Dom::QmlFile>();
     const auto code = qmlFile->code();
     const int startOffset = int(QQmlLSUtils::textOffsetFrom(code, range.start.line, range.end.character));
     const int endOffset = int(QQmlLSUtils::textOffsetFrom(code, range.end.line, range.end.character));
-    const auto expectedData = HighlightingUtils::collectTokens(item, HighlightsRange{startOffset, endOffset});
+    const auto expectedData = QmlHighlighting::Utils::collectTokens(
+            item, QmlHighlighting::HighlightsRange{ startOffset, endOffset });
 
     const auto uri = openFile(filePath);
     QVERIFY(uri);
@@ -1884,9 +1884,10 @@ void tst_qmlls_modules::semanticHighlightingDelta()
 
     const auto fileItem = fileObject(testFile(filePath));
     const auto deltaFileItem = fileObject(testFile(deltaFilePath));
-    auto fullDocumentSemanticTokensData = HighlightingUtils::collectTokens(fileItem, std::nullopt);
-    auto editedDocumentSemanticTokensData = HighlightingUtils::collectTokens(deltaFileItem, std::nullopt);
-    const auto expectedEdits = HighlightingUtils::computeDiff(fullDocumentSemanticTokensData, editedDocumentSemanticTokensData);
+    auto fullDocumentSemanticTokensData = QmlHighlighting::Utils::collectTokens(fileItem, std::nullopt);
+    auto editedDocumentSemanticTokensData = QmlHighlighting::Utils::collectTokens(deltaFileItem, std::nullopt);
+    const auto expectedEdits = QmlHighlighting::Utils::computeDiff(
+            fullDocumentSemanticTokensData, editedDocumentSemanticTokensData);
 
     const auto uri = openFile(filePath);
     QVERIFY(uri);
