@@ -1,7 +1,8 @@
 // Copyright (C) 2018 The Qt Company Ltd.
 // SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
-#include <QtTest/QtTest>
+#include <QtTest/QTest>
+#include <QtTest/QSignalSpy>
 
 #include <QtQuick/qquickview.h>
 #include <QtQuick/qquickitem.h>
@@ -50,11 +51,12 @@ private slots:
     void ensureHoverHandlerWorksWhenItemHasHoverDisabled();
     void changeCursor();
     void touchDrag();
+    void asProperty();
 
 private:
     void createView(QScopedPointer<QQuickView> &window, const char *fileName);
 
-    QScopedPointer<QPointingDevice> touchscreen = QScopedPointer<QPointingDevice>(QTest::createTouchDevice());
+    std::unique_ptr<QPointingDevice> touchscreen{QTest::createTouchDevice()};
 };
 
 void tst_HoverHandler::createView(QScopedPointer<QQuickView> &window, const char *fileName)
@@ -394,14 +396,25 @@ void tst_HoverHandler::movingItemWithHoverHandler()
     QTRY_COMPARE(paddleHH->isHovered(), true);
     // TODO check the cursor shape after fixing QTBUG-53987
 
+    const auto &deliveryTargets =
+            QQuickPointerHandlerPrivate::deviceDeliveryTargets(QPointingDevice::primaryPointingDevice());
+    const auto targetsCount = deliveryTargets.size();
+    qCDebug(lcPointerTests) << "deviceDeliveryTargets before paddle movement" << deliveryTargets;
     paddle->setX(100);
     QTRY_COMPARE(paddleHH->isHovered(), false);
+    // QQuickDeliveryAgentPrivate::deliverHoverEvent() clears the deviceDeliveryTargets list,
+    // and then each HoverHandler's QQuickPointerHandler::handlePointerEvent() adds itself again.
+    // As long as we visit the same handlers each time, the list should not grow. (QTBUG-135975)
+    qCDebug(lcPointerTests) << "deviceDeliveryTargets after paddle movement" << deliveryTargets;
+    QCOMPARE(deliveryTargets.size(), targetsCount);
 
     paddle->setX(p.x() - paddle->width() / 2);
     QTRY_COMPARE(paddleHH->isHovered(), true);
+    QCOMPARE(deliveryTargets.size(), targetsCount);
 
     paddle->setX(540);
     QTRY_COMPARE(paddleHH->isHovered(), false);
+    QCOMPARE(deliveryTargets.size(), targetsCount);
 }
 
 void tst_HoverHandler::margin() // QTBUG-85303
@@ -458,10 +471,13 @@ void tst_HoverHandler::window() // QTBUG-98717
 {
     QQmlEngine engine;
     QQmlComponent component(&engine);
+    const QPoint pos(30, 30);
     component.loadUrl(testFileUrl("windowCursorShape.qml"));
     QScopedPointer<QQuickWindow> window(qobject_cast<QQuickWindow *>(component.create()));
     QVERIFY(!window.isNull());
+    window->setFramePosition(pos);
     window->show();
+    QTRY_COMPARE(window->framePosition(), pos);
     QVERIFY(QTest::qWaitForWindowExposed(window.data()));
 #if QT_CONFIG(cursor)
     if (isPlatformWayland())
@@ -736,7 +752,7 @@ void tst_HoverHandler::touchDrag()
     QTest::touchEvent(&window, touchscreen.get()).move(0, in, &window);
     QQuickTouchUtils::flush(&window);
     QTRY_COMPARE(handler->isHovered(), true);
-    QCOMPARE(handler->point().scenePosition(), in);
+    QCOMPARE(handler->point().scenePosition().toPoint(), in);
 
     in += {10, 10};
     QTest::touchEvent(&window, touchscreen.get()).move(0, in, &window);
@@ -744,7 +760,7 @@ void tst_HoverHandler::touchDrag()
     // ensure that the color change is visible
     QTRY_COMPARE_GE(frameSyncSpy.size(), 1);
     QCOMPARE(handler->isHovered(), true);
-    QCOMPARE(handler->point().scenePosition(), in);
+    QCOMPARE(handler->point().scenePosition().toPoint(), in);
 
     QTest::touchEvent(&window, touchscreen.get()).move(0, out, &window);
     QQuickTouchUtils::flush(&window);
@@ -754,6 +770,19 @@ void tst_HoverHandler::touchDrag()
     QTest::touchEvent(&window, touchscreen.get()).release(0, out, &window);
 }
 
+void tst_HoverHandler::asProperty()
+{
+    QQuickView window;
+    window.setFlag(Qt::FramelessWindowHint, true);
+    QVERIFY(QQuickTest::showView(window, testFileUrl("asProperty.qml")));
+    const QQuickItem *root = window.rootObject();
+    QQuickHoverHandler *handler = root->property("handler").value<QQuickHoverHandler *>();
+    QVERIFY(handler);
+    QCOMPARE(handler->isHovered(), false);
+    QTest::mouseMove(&window, root->boundingRect().center().toPoint());
+    QTRY_COMPARE(handler->isHovered(), true);
+}
+
 QTEST_MAIN(tst_HoverHandler)
 
 #include "tst_qquickhoverhandler.moc"
