@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 #include <qtest.h>
 #include <QtQml/qqmlengine.h>
 #include <QtQml/qqmlexpression.h>
@@ -77,9 +52,9 @@ private slots:
     void remote();
     void remote_data();
     void sourceSize();
+    void setSourceSize();
     void sourceSizeChanges();
     void sourceSizeChanges_intermediate();
-    void sourceSizeReadOnly();
     void invalidSource();
     void qtbug_16520();
     void progressAndStatusChanges();
@@ -87,6 +62,7 @@ private slots:
     void noCaching();
     void sourceChangesOnFrameChanged();
     void currentFrame();
+    void qtbug_120555();
 };
 
 void tst_qquickanimatedimage::cleanup()
@@ -157,10 +133,10 @@ void tst_qquickanimatedimage::frameCount()
     const QUrl origSource = anim->source();
     anim->setSource(QUrl());
     QCOMPARE(anim->frameCount(), 0);
-    QCOMPARE(frameCountChangedSpy.count(), 1);
+    QCOMPARE(frameCountChangedSpy.size(), 1);
     anim->setSource(origSource);
     QCOMPARE(anim->frameCount(), 3);
-    QCOMPARE(frameCountChangedSpy.count(), 2);
+    QCOMPARE(frameCountChangedSpy.size(), 2);
 
     delete anim;
 }
@@ -194,13 +170,13 @@ void tst_qquickanimatedimage::mirror_running()
     QVERIFY(spy.isValid());
     anim->setPlaying(true);
 
-    QTRY_VERIFY(spy.count() == 1); spy.clear();
+    QTRY_VERIFY(spy.size() == 1); spy.clear();
     anim->setMirror(true);
 
     QCOMPARE(anim->currentFrame(), 1);
     QImage frame1_flipped = window.grabWindow();
 
-    QTRY_VERIFY(spy.count() == 1); spy.clear();
+    QTRY_VERIFY(spy.size() == 1); spy.clear();
     QCOMPARE(anim->currentFrame(), 0);  // animation only has 2 frames, should cycle back to first
     QImage frame0_flipped = window.grabWindow();
 
@@ -312,12 +288,19 @@ void tst_qquickanimatedimage::sourceSize()
     delete anim;
 }
 
-void tst_qquickanimatedimage::sourceSizeReadOnly()
+void tst_qquickanimatedimage::setSourceSize()
 {
     QQmlEngine engine;
-    QQmlComponent component(&engine, testFileUrl("stickmanerror1.qml"));
-    QVERIFY(component.isError());
-    QCOMPARE(component.errors().at(0).description(), QString("Invalid property assignment: \"sourceSize\" is a read-only property"));
+    QQmlComponent component(&engine, testFileUrl("stickmansourcesized.qml"));
+    QScopedPointer<QQuickAnimatedImage> anim(qobject_cast<QQuickAnimatedImage *>(component.create()));
+    QVERIFY(anim);
+    QCOMPARE(anim->sourceSize(), QSize(80, 60));
+
+    anim->setSourceSize(QSize(40, 30));
+    QCOMPARE(anim->sourceSize(), QSize(40, 30));
+
+    anim->setSourceSize(QSize());
+    QCOMPARE(anim->sourceSize(), QSize(160, 120));
 }
 
 void tst_qquickanimatedimage::invalidSource()
@@ -361,48 +344,48 @@ void tst_qquickanimatedimage::sourceSizeChanges()
     // Local
     ctxt->setContextProperty("srcImage", QUrl(""));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Null);
-    QTRY_COMPARE(sourceSizeSpy.count(), 0);
+    QTRY_COMPARE(sourceSizeSpy.size(), 0);
 
     ctxt->setContextProperty("srcImage", testFileUrl("hearts.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_COMPARE(sourceSizeSpy.count(), 1);
+    QTRY_COMPARE(sourceSizeSpy.size(), 1);
 
     ctxt->setContextProperty("srcImage", testFileUrl("hearts.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_COMPARE(sourceSizeSpy.count(), 1);
+    QTRY_COMPARE(sourceSizeSpy.size(), 1);
 
     ctxt->setContextProperty("srcImage", testFileUrl("hearts_copy.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_COMPARE(sourceSizeSpy.count(), 1);
+    QTRY_COMPARE(sourceSizeSpy.size(), 1);
 
     ctxt->setContextProperty("srcImage", testFileUrl("colors.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_COMPARE(sourceSizeSpy.count(), 2);
+    QTRY_COMPARE(sourceSizeSpy.size(), 2);
 
     ctxt->setContextProperty("srcImage", QUrl(""));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Null);
-    QTRY_COMPARE(sourceSizeSpy.count(), 3);
+    QTRY_COMPARE(sourceSizeSpy.size(), 3);
 
     // Remote
     ctxt->setContextProperty("srcImage", server.url("/hearts.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_COMPARE(sourceSizeSpy.count(), 4);
+    QTRY_COMPARE(sourceSizeSpy.size(), 4);
 
     ctxt->setContextProperty("srcImage", server.url("/hearts.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_COMPARE(sourceSizeSpy.count(), 4);
+    QTRY_COMPARE(sourceSizeSpy.size(), 4);
 
     ctxt->setContextProperty("srcImage", server.url("/hearts_copy.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_COMPARE(sourceSizeSpy.count(), 4);
+    QTRY_COMPARE(sourceSizeSpy.size(), 4);
 
     ctxt->setContextProperty("srcImage", server.url("/colors.gif"));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Ready);
-    QTRY_COMPARE(sourceSizeSpy.count(), 5);
+    QTRY_COMPARE(sourceSizeSpy.size(), 5);
 
     ctxt->setContextProperty("srcImage", QUrl(""));
     QTRY_COMPARE(anim->status(), QQuickAnimatedImage::Null);
-    QTRY_COMPARE(sourceSizeSpy.count(), 6);
+    QTRY_COMPARE(sourceSizeSpy.size(), 6);
 
     delete anim;
 }
@@ -478,17 +461,17 @@ void tst_qquickanimatedimage::progressAndStatusChanges()
     ctxt->setContextProperty("srcImage", testFileUrl("stickman.gif"));
     QTRY_COMPARE(obj->status(), QQuickImage::Ready);
     QTRY_COMPARE(obj->progress(), 1.0);
-    QTRY_COMPARE(sourceSpy.count(), 0);
-    QTRY_COMPARE(progressSpy.count(), 0);
-    QTRY_COMPARE(statusSpy.count(), 0);
+    QTRY_COMPARE(sourceSpy.size(), 0);
+    QTRY_COMPARE(progressSpy.size(), 0);
+    QTRY_COMPARE(statusSpy.size(), 0);
 
     // Loading local file
     ctxt->setContextProperty("srcImage", testFileUrl("colors.gif"));
     QTRY_COMPARE(obj->status(), QQuickImage::Ready);
     QTRY_COMPARE(obj->progress(), 1.0);
-    QTRY_COMPARE(sourceSpy.count(), 1);
-    QTRY_COMPARE(progressSpy.count(), 0);
-    QTRY_COMPARE(statusSpy.count(), 1);
+    QTRY_COMPARE(sourceSpy.size(), 1);
+    QTRY_COMPARE(progressSpy.size(), 0);
+    QTRY_COMPARE(statusSpy.size(), 1);
 
     // Loading remote file
     ctxt->setContextProperty("srcImage", server.url("/stickman.gif"));
@@ -496,16 +479,16 @@ void tst_qquickanimatedimage::progressAndStatusChanges()
     QTRY_COMPARE(obj->progress(), 0.0);
     QTRY_COMPARE(obj->status(), QQuickImage::Ready);
     QTRY_COMPARE(obj->progress(), 1.0);
-    QTRY_COMPARE(sourceSpy.count(), 2);
-    QTRY_VERIFY(progressSpy.count() > 1);
-    QTRY_COMPARE(statusSpy.count(), 3);
+    QTRY_COMPARE(sourceSpy.size(), 2);
+    QTRY_VERIFY(progressSpy.size() > 1);
+    QTRY_COMPARE(statusSpy.size(), 3);
 
     ctxt->setContextProperty("srcImage", "");
     QTRY_COMPARE(obj->status(), QQuickImage::Null);
     QTRY_COMPARE(obj->progress(), 0.0);
-    QTRY_COMPARE(sourceSpy.count(), 3);
-    QTRY_VERIFY(progressSpy.count() > 2);
-    QTRY_COMPARE(statusSpy.count(), 4);
+    QTRY_COMPARE(sourceSpy.size(), 3);
+    QTRY_VERIFY(progressSpy.size() > 2);
+    QTRY_COMPARE(statusSpy.size(), 4);
 
     delete obj;
 }
@@ -531,40 +514,40 @@ void tst_qquickanimatedimage::playingAndPausedChanges()
     obj->setProperty("paused", false);
     QTRY_VERIFY(obj->isPlaying());
     QTRY_VERIFY(!obj->isPaused());
-    QTRY_COMPARE(playingSpy.count(), 0);
-    QTRY_COMPARE(pausedSpy.count(), 0);
+    QTRY_COMPARE(playingSpy.size(), 0);
+    QTRY_COMPARE(pausedSpy.size(), 0);
 
     obj->setProperty("playing", false);
     obj->setProperty("paused", true);
     QTRY_VERIFY(!obj->isPlaying());
     QTRY_VERIFY(obj->isPaused());
-    QTRY_COMPARE(playingSpy.count(), 1);
-    QTRY_COMPARE(pausedSpy.count(), 1);
+    QTRY_COMPARE(playingSpy.size(), 1);
+    QTRY_COMPARE(pausedSpy.size(), 1);
 
     obj->setProperty("playing", true);
     obj->setProperty("paused", false);
     QTRY_VERIFY(obj->isPlaying());
     QTRY_VERIFY(!obj->isPaused());
-    QTRY_COMPARE(playingSpy.count(), 2);
-    QTRY_COMPARE(pausedSpy.count(), 2);
+    QTRY_COMPARE(playingSpy.size(), 2);
+    QTRY_COMPARE(pausedSpy.size(), 2);
 
     ctxt->setContextProperty("srcImage", testFileUrl("stickman.gif"));
     QTRY_VERIFY(obj->isPlaying());
     QTRY_VERIFY(!obj->isPaused());
-    QTRY_COMPARE(playingSpy.count(), 2);
-    QTRY_COMPARE(pausedSpy.count(), 2);
+    QTRY_COMPARE(playingSpy.size(), 2);
+    QTRY_COMPARE(pausedSpy.size(), 2);
 
     obj->setProperty("paused", true);
     QTRY_VERIFY(obj->isPlaying());
     QTRY_VERIFY(obj->isPaused());
-    QTRY_COMPARE(playingSpy.count(), 2);
-    QTRY_COMPARE(pausedSpy.count(), 3);
+    QTRY_COMPARE(playingSpy.size(), 2);
+    QTRY_COMPARE(pausedSpy.size(), 3);
 
     obj->setProperty("playing", false);
     QTRY_VERIFY(!obj->isPlaying());
     QTRY_VERIFY(!obj->isPaused());
-    QTRY_COMPARE(playingSpy.count(), 3);
-    QTRY_COMPARE(pausedSpy.count(), 4);
+    QTRY_COMPARE(playingSpy.size(), 3);
+    QTRY_COMPARE(pausedSpy.size(), 4);
 
     obj->setProperty("playing", true);
 
@@ -572,8 +555,8 @@ void tst_qquickanimatedimage::playingAndPausedChanges()
     ctxt->setContextProperty("srcImage", testFileUrl("green.png"));
     QTRY_VERIFY(!obj->isPlaying());
     QTRY_VERIFY(!obj->isPaused());
-    QTRY_COMPARE(playingSpy.count(), 5);
-    QTRY_COMPARE(pausedSpy.count(), 4);
+    QTRY_COMPARE(playingSpy.size(), 5);
+    QTRY_COMPARE(pausedSpy.size(), 4);
 
     delete obj;
 }
@@ -651,19 +634,49 @@ void tst_qquickanimatedimage::currentFrame()
 
     anim->setCurrentFrame(1);
     QCOMPARE(anim->currentFrame(), 1);
-    QCOMPARE(frameChangedSpy.count(), 1);
-    QCOMPARE(currentFrameChangedSpy.count(), 1);
+    QCOMPARE(frameChangedSpy.size(), 1);
+    QCOMPARE(currentFrameChangedSpy.size(), 1);
     QCOMPARE(anim->property("currentFrameChangeCount"), 1);
     QCOMPARE(anim->property("frameChangeCount"), 1);
 
     evaluate<void>(anim, "scriptedSetCurrentFrame(2)");
     QCOMPARE(anim->currentFrame(), 2);
-    QCOMPARE(frameChangedSpy.count(), 2);
-    QCOMPARE(currentFrameChangedSpy.count(), 2);
+    QCOMPARE(frameChangedSpy.size(), 2);
+    QCOMPARE(currentFrameChangedSpy.size(), 2);
     QCOMPARE(anim->property("currentFrameChangeCount"), 2);
     QCOMPARE(anim->property("frameChangeCount"), 2);
 }
 
+void tst_qquickanimatedimage::qtbug_120555()
+{
+    TestHTTPServer server;
+    QVERIFY2(server.listen(), qPrintable(server.errorString()));
+    server.serveDirectory(dataDirectory());
+
+    QQmlEngine engine;
+    QQmlComponent component(&engine);
+    component.setData("import QtQuick 2.0\nAnimatedImage {}", {});
+
+    QQuickAnimatedImage *anim = qobject_cast<QQuickAnimatedImage*>(component.create());
+    QVERIFY(anim);
+
+    anim->setSource(server.url("/stickman.gif"));
+    QTRY_COMPARE(anim->status(), QQuickImage::Loading);
+
+    anim->setFillMode(QQuickImage::PreserveAspectFit);
+    QCOMPARE(anim->fillMode(), QQuickImage::PreserveAspectFit);
+    anim->setMipmap(true);
+    QCOMPARE(anim->mipmap(), true);
+    anim->setCache(false);
+    QCOMPARE(anim->cache(), false);
+    anim->setSourceSize(QSize(200, 200));
+    QCOMPARE(anim->sourceSize(), QSize(200, 200));
+
+    QTRY_COMPARE(anim->status(), QQuickImage::Ready);
+
+    delete anim;
+}
+
 QTEST_MAIN(tst_qquickanimatedimage)
 
 #include "tst_qquickanimatedimage.moc"
