@@ -5,6 +5,7 @@
 
 #include <QtQml/QQmlComponent>
 #include <QtQuick/qquickitem.h>
+#include <QtQuick/qquickitemgrabresult.h>
 #include <QtQuick/qquickwindow.h>
 #include <QtQuick/qquickview.h>
 #include "private/qquickfocusscope_p.h"
@@ -18,6 +19,7 @@
 #include <QDebug>
 #include <QTimer>
 #include <QQmlEngine>
+#include <QtQuickTest/quicktest.h>
 #include <QtQuickTestUtils/private/qmlutils_p.h>
 #include <QtQuickTestUtils/private/viewtestutils_p.h>
 #include <QSignalSpy>
@@ -232,6 +234,8 @@ private slots:
 
     void transformChanged();
 
+    void grabImage();
+
 private:
 
     enum PaintOrderOp {
@@ -245,6 +249,8 @@ private:
         w->requestActivate();
         return QTest::qWaitForWindowActive(w);
     }
+
+    std::unique_ptr<QPointingDevice> touchscreen{QTest::createTouchDevice()};
 };
 
 tst_qquickitem::tst_qquickitem()
@@ -809,13 +815,13 @@ void tst_qquickitem::focusSubItemInNonFocusScope()
 
     QVERIFY(dummyItem->hasFocus());
     QVERIFY(!textInput->hasFocus());
-    QVERIFY(dummyItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(dummyItem);
 
     QVERIFY(QMetaObject::invokeMethod(textInput, "forceActiveFocus"));
 
     QVERIFY(!dummyItem->hasFocus());
     QVERIFY(textInput->hasFocus());
-    QVERIFY(textInput->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(textInput);
 }
 
 void tst_qquickitem::parentItemWithFocus()
@@ -1459,21 +1465,19 @@ void tst_qquickitem::touchEventAcceptIgnore()
         item->setAcceptTouchEvents(itemAcceptsTouch); // it's false by default in Qt 6
     item->acceptIncomingTouchEvents = itemAcceptsTouchEvents;
 
-    static QPointingDevice* device = QTest::createTouchDevice();
-
     // Send Begin, Update & End touch sequence
     item->touchEventReached = false;
-    QTest::touchEvent(&window, device).press(1, QPoint(50, 50), &window);
+    QTest::touchEvent(&window, touchscreen.get()).press(1, QPoint(50, 50), &window);
     QQuickTouchUtils::flush(&window);
     QTRY_COMPARE(item->touchEventReached, itemAcceptsTouch);
 
     item->touchEventReached = false;
-    QTest::touchEvent(&window, device).move(1, QPoint(60, 60), &window);
+    QTest::touchEvent(&window, touchscreen.get()).move(1, QPoint(60, 60), &window);
     QQuickTouchUtils::flush(&window);
     QTRY_COMPARE(item->touchEventReached, itemAcceptsTouchEvents);
 
     item->touchEventReached = false;
-    QTest::touchEvent(&window, device).release(1, QPoint(60, 60), &window);
+    QTest::touchEvent(&window, touchscreen.get()).release(1, QPoint(60, 60), &window);
     QQuickTouchUtils::flush(&window);
     QTRY_COMPARE(item->touchEventReached, itemAcceptsTouchEvents);
 }
@@ -2312,7 +2316,7 @@ void tst_qquickitem::shortcutOverride()
 
     QQuickItem *escapeItem = view.rootObject()->property("escapeItem").value<QQuickItem*>();
     QVERIFY(escapeItem);
-    QVERIFY(escapeItem->hasActiveFocus());
+    QVERIFY_ACTIVE_FOCUS(escapeItem);
 
     // escapeItem's onEscapePressed handler should accept the first escape press event.
     QTest::keyPress(&view, Qt::Key_Escape);
@@ -2677,6 +2681,21 @@ void tst_qquickitem::transformChanged()
     QCOMPARE(transformItem.mapToScene(QPoint(0, 0)), parents[1][0]->position());
 }
 
+void tst_qquickitem::grabImage()
+{
+    QQmlEngine engine;
+    QQmlComponent component(&engine, testFileUrl("grabImage.qml"));
+
+    QScopedPointer<QQuickWindow> window(qobject_cast<QQuickWindow*>(component.create()));
+    QVERIFY(window);
+
+    gc(engine);
+
+    QTRY_VERIFY(window->property("finishedSuccessfuly").toBool());
+    QQuickItemGrabResult *result = window->property("itemGrabResult").value<QQuickItemGrabResult*>();
+    QVERIFY(result);
+}
+
 QTEST_MAIN(tst_qquickitem)
 
 #include "tst_qquickitem.moc"
