@@ -1,36 +1,15 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtQuick module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <QtTest/qtest.h>
 #include <QtQuick/qquickwindow.h>
 #include <QtQml/qqmlapplicationengine.h>
 #include <QtQuick/qquickitem.h>
+#include <QtQuick/qquickview.h>
 #include <QtTest/qsignalspy.h>
+#ifdef QT_WIDGETS_LIB
+#include <QtWidgets/qboxlayout.h>
+#endif
 #ifdef QT_QUICKWIDGETS_LIB
 #include <QtQuickWidgets/qquickwidget.h>
 #endif
@@ -59,6 +38,8 @@ private slots:
     void matcher();
     void multiple_data();
     void multiple();
+    void embedded_data();
+    void embedded();
 #ifdef QT_QUICKWIDGETS_LIB
     void quickWidgetShortcuts_data();
     void quickWidgetShortcuts();
@@ -125,13 +106,25 @@ void tst_QQuickShortcut::standardShortcuts()
     shortcuts.push_back(standardKey);
     shortcut->setProperty("sequences", shortcuts);
 
-    // test all:
+
+    /* test that all keys trigger the shortcut, and also
+       test that we get a sensible value for nativeText
+       and portableText */
+    const QString nativeText = shortcut->property("nativeText").toString();
+    bool foundMatchingNativeText = false;
+    const QString portableText = shortcut->property("portableText").toString();
+    bool foundMatchingPortableText = false;
     QList<QKeySequence> allsequences = QKeySequence::keyBindings(standardKey);
     for (const QKeySequence &s : allsequences) {
+        foundMatchingNativeText |= s.toString(QKeySequence::NativeText) == nativeText;
+        foundMatchingPortableText |= s.toString(QKeySequence::PortableText) == portableText;
         window->setProperty("activated", false);
         QTest::keySequence(window, s);
         QCOMPARE(window->property("activated").toBool(), true);
     }
+
+    QVERIFY(foundMatchingNativeText);
+    QVERIFY(foundMatchingPortableText);
 }
 
 void tst_QQuickShortcut::shortcuts_data()
@@ -423,9 +416,14 @@ void tst_QQuickShortcut::context()
 }
 
 typedef bool (*ShortcutContextMatcher)(QObject *, Qt::ShortcutContext);
+
+QT_BEGIN_NAMESPACE
+
 extern ShortcutContextMatcher qt_quick_shortcut_context_matcher();
 extern void qt_quick_set_shortcut_context_matcher(ShortcutContextMatcher matcher);
 
+QT_END_NAMESPACE
+
 static ShortcutContextMatcher lastMatcher = nullptr;
 
 static bool trueMatcher(QObject *, Qt::ShortcutContext)
@@ -602,6 +600,140 @@ void tst_QQuickShortcut::contextChange()
     QCOMPARE(inactiveWindow->property("activated").toBool(), activated);
 }
 
+void tst_QQuickShortcut::embedded_data()
+{
+    QTest::addColumn<Qt::Key>("testKey");
+    QTest::addColumn<Qt::KeyboardModifiers>("testModifiers");
+    QTest::addColumn<QString>("windowShortcutSequence");
+    QTest::addColumn<QString>("applicationShortcutSequence");
+    QTest::addColumn<bool>("windowShortcutActivated");
+    QTest::addColumn<bool>("applicationShortcutActivated");
+
+    QTest::newRow("windowActivated") << Qt::Key_W << (Qt::ControlModifier | Qt::AltModifier)
+                                     << "Ctrl+Alt+W" << "Ctrl+Alt+A" << true << false;
+    QTest::newRow("applicationActivated") << Qt::Key_A << (Qt::ControlModifier | Qt::AltModifier)
+                                          << "Ctrl+Alt+W" << "Ctrl+Alt+A" << false << true;
+}
+
+void tst_QQuickShortcut::embedded()
+{
+#ifndef QT_WIDGETS_LIB
+    QSKIP("Skipping due to QT_WIDGETS_LIB is not defined");
+#else
+    QFETCH(Qt::Key, testKey);
+    QFETCH(Qt::KeyboardModifiers, testModifiers);
+    QFETCH(QString, windowShortcutSequence);
+    QFETCH(QString, applicationShortcutSequence);
+    QFETCH(bool, windowShortcutActivated);
+    QFETCH(bool, applicationShortcutActivated);
+
+    QWidget window;
+    QVBoxLayout *layout = new QVBoxLayout {&window};
+    QQuickView *quickView = new QQuickView;
+    quickView->setResizeMode(QQuickView::SizeRootObjectToView);
+    quickView->setSource(testFileUrl("embedded.qml"));
+
+    QWidget *container = QWidget::createWindowContainer(quickView);
+    container->setMinimumSize(quickView->size());
+    container->setFocusPolicy(Qt::TabFocus);
+
+    QWidget *widget = new QWidget;
+    // We will set focus to the widget and its default is NoFocus.
+    widget->setFocusPolicy(Qt::FocusPolicy::StrongFocus);
+
+    layout->addWidget(widget);
+    layout->addWidget(container);
+
+    window.show();
+    QTRY_VERIFY(window.isVisible());
+    // The widget can get focused only when the including window has exposed.
+    QVERIFY(QTest::qWaitForWindowExposed(&window));
+
+    QWindow *windowHandle = window.windowHandle();
+    windowHandle->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(windowHandle));
+
+    widget->setFocus();
+    QTRY_VERIFY(widget->hasFocus());
+
+    // If the window-context shortcut is expected to be activated,
+    // then the QuickView window needs to be active.
+    // On Linux, the embedded QuickView window is active immediately
+    // after the containing window is active, but on Windows,
+    // the embedded QuickView window is activated after a delay
+    // once the containing window is active.
+    if (windowShortcutActivated)
+        QVERIFY(QTest::qWaitForWindowActive(quickView));
+
+    QQuickItem *item = quickView->rootObject();
+    QVERIFY(item);
+
+    QObject *windowShortcut = item->property("shortcut").value<QObject *>();
+    QVERIFY(windowShortcut);
+
+    windowShortcut->setProperty("context", Qt::WindowShortcut);
+    windowShortcut->setProperty("sequence", windowShortcutSequence);
+
+    QTest::keyPress(&window, testKey, testModifiers);
+    QTest::keyRelease(&window, testKey, testModifiers);
+    QCOMPARE(item->property("activated").toBool(), windowShortcutActivated);
+
+    quickView->requestActivate();
+    QTRY_VERIFY(quickView->isActive());
+    QVERIFY(quickView->isActive());
+
+    item->setProperty("activated", false);
+    QVERIFY(!item->property("activated").toBool());
+
+    QTest::keyPress(&window, testKey, testModifiers);
+    QTest::keyRelease(&window, testKey, testModifiers);
+    QCOMPARE(item->property("activated").toBool(), windowShortcutActivated);
+
+    QWidget otherWindow;
+    QVBoxLayout *otherLayout = new QVBoxLayout {&otherWindow};
+    QQuickView *otherQuickView = new QQuickView;
+    otherQuickView->setResizeMode(QQuickView::SizeRootObjectToView);
+    otherQuickView->setSource(testFileUrl("embedded.qml"));
+
+    QWidget *otherContainer = QWidget::createWindowContainer(otherQuickView);
+    otherContainer->setMinimumSize(quickView->size());
+    otherContainer->setFocusPolicy(Qt::TabFocus);
+
+    QWidget *otherWidget = new QWidget;
+    otherLayout->addWidget(otherWidget);
+    otherLayout->addWidget(otherContainer);
+
+    otherWindow.show();
+    QTRY_VERIFY(otherWindow.isVisible());
+    QVERIFY(otherWindow.isVisible());
+
+    // make sure that the (first) window is active
+    quickView->requestActivate();
+    QVERIFY(QTest::qWaitForWindowActive(quickView));
+
+    QQuickItem *otherItem = otherQuickView->rootObject();
+    QVERIFY(otherItem);
+
+    QObject *applicationShortcut = otherItem->property("shortcut").value<QObject *>();
+    QVERIFY(applicationShortcut);
+
+    applicationShortcut->setProperty("context", Qt::ApplicationShortcut);
+    applicationShortcut->setProperty("sequence", applicationShortcutSequence);
+
+    QTest::keyPress(&otherWindow, testKey, testModifiers);
+    QTest::keyRelease(&otherWindow, testKey, testModifiers);
+    QCOMPARE(otherItem->property("activated").toBool(), applicationShortcutActivated);
+
+    otherItem->setProperty("activated", false);
+    otherWindow.close();
+    QTRY_VERIFY(!otherWindow.isVisible());
+
+    QTest::keyPress(&otherWindow, testKey, testModifiers);
+    QTest::keyRelease(&otherWindow, testKey, testModifiers);
+    QCOMPARE(otherItem->property("activated").toBool(), false);
+#endif
+}
+
 #ifdef QT_QUICKWIDGETS_LIB
 void tst_QQuickShortcut::quickWidgetShortcuts_data()
 {
