@@ -62,38 +62,53 @@ void tst_ObjectCount::cleanup()
 
 static void initTestRows(QQmlEngine *engine)
 {
-    // Calendar is excluded because it's a singleton and can't be created.
-    // TreeViewDelegate is excluded since it's a delegate that can only be created by TreeView.
-    // TableViewDelegate is excluded since it's a delegate that can only be created by TableView.
-    // Horizontal/VerticalHeaderViewDelegate is excluded since it's a delegate
-    // that can only be created by Horizontal/VerticalHeaderView.
+    const QStringList commonExclusions = {
+        // Singleton; can't be created.
+        "Calendar",
+        // Can only be created by HorizontalHeaderView.
+        "HorizontalHeaderViewDelegate",
+        // Can only be created by TreeView.
+        "TreeViewDelegate",
+        // Can only be created by TableView.
+        "TableViewDelegate",
+        // Can only be created by VerticalHeaderView.
+        "VerticalHeaderViewDelegate"
+    };
+
     addTestRowForEachControl(engine, QQC2_IMPORT_PATH, "basic", "QtQuick/Controls/Basic",
-                             QStringList() << "Calendar"
-                                           << "TreeViewDelegate"
-                                           << "TableViewDelegate"
-                                           << "HorizontalHeaderViewDelegate"
-                                           << "VerticalHeaderViewDelegate");
-    addTestRowForEachControl(
-            engine, QQC2_IMPORT_PATH, "fusion", "QtQuick/Controls/Fusion",
-            QStringList() << "ButtonPanel" << "CheckIndicator" << "RadioIndicator" << "SliderGroove"
-                          << "SliderHandle" << "SwitchIndicator" << "TreeViewDelegate"
-                          << "HorizontalHeaderViewDelegate"
-                          << "VerticalHeaderViewDelegate");
+        commonExclusions);
+
+    addTestRowForEachControl(engine, QQC2_IMPORT_PATH, "fusion", "QtQuick/Controls/Fusion",
+        QStringList()
+            << commonExclusions
+            << "ButtonPanel"
+            << "CheckIndicator"
+            << "RadioIndicator"
+            << "SliderGroove"
+            << "SliderHandle"
+            << "SwitchIndicator");
+
     addTestRowForEachControl(engine, QQC2_IMPORT_PATH, "imagine", "QtQuick/Controls/Imagine",
-                             QStringList() << "HorizontalHeaderViewDelegate"
-                                           << "VerticalHeaderViewDelegate");
+        commonExclusions);
+
     addTestRowForEachControl(engine, QQC2_IMPORT_PATH, "material", "QtQuick/Controls/Material",
-                             QStringList() << "Ripple" << "SliderHandle" << "CheckIndicator"
-                                           << "RadioIndicator"
-                                           << "SwitchIndicator" << "BoxShadow" << "ElevationEffect"
-                                           << "CursorDelegate" << "TreeViewDelegate"
-                                           << "HorizontalHeaderViewDelegate"
-                                           << "VerticalHeaderViewDelegate");
+        QStringList()
+           << commonExclusions
+           << "BoxShadow"
+           << "CheckIndicator"
+           << "CursorDelegate"
+           << "ElevationEffect"
+           << "RadioIndicator"
+           << "Ripple"
+           << "SliderHandle"
+           << "SwitchIndicator");
+
     addTestRowForEachControl(engine, QQC2_IMPORT_PATH, "universal", "QtQuick/Controls/Universal",
-                             QStringList()
-                                     << "CheckIndicator" << "RadioIndicator" << "SwitchIndicator"
-                                     << "HorizontalHeaderViewDelegate"
-                                     << "VerticalHeaderViewDelegate");
+        QStringList()
+            << commonExclusions
+            << "CheckIndicator"
+            << "RadioIndicator"
+            << "SwitchIndicator");
 }
 
 template <typename T>
